@@ -3,8 +3,8 @@
 /*
 	Phoronix Test Suite
 	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
-	Copyright (C) 2008 - 2014, Phoronix Media
-	Copyright (C) 2008 - 2014, Michael Larabel
+	Copyright (C) 2008 - 2021, Phoronix Media
+	Copyright (C) 2008 - 2021, Michael Larabel
 	toggle_screensaver.php: A module to toggle the screensaver while tests are running on GNOME
 
 	This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,7 @@
 class toggle_screensaver extends pts_module_interface
 {
 	const module_name = 'Toggle Screensaver';
-	const module_version = '1.5.0';
+	const module_version = '1.5.2';
 	const module_description = 'This module toggles the system\'s screensaver while the Phoronix Test Suite is running. At this time, the GNOME and KDE screensavers are supported.';
 	const module_author = 'Phoronix Media';
 
@@ -33,6 +33,7 @@ class toggle_screensaver extends pts_module_interface
 	static $screensaver_halted = false;
 	static $gnome2_screensaver_halted = false;
 	static $gnome3_screensaver_halted = false;
+	static $gnome3_lockscreen_disabled = false;
 	static $gnome3_screensaver_halted_old = false;
 	static $kde_screensaver_halted = false;
 	static $gnome_gconftool = false;
@@ -45,12 +46,16 @@ class toggle_screensaver extends pts_module_interface
 	}
 	public static function __startup()
 	{
-		$halt_screensaver = pts_module::read_variable('HALT_SCREENSAVER');
-		if((!empty($halt_screensaver) && !pts_strings::string_bool($halt_screensaver)) || phodevi::read_property('system', 'display-server') == null)
+		if(getenv('PTS_SYSTEMD_INHIBITED') != false)
 		{
 			return pts_module::MODULE_UNLOAD;
 		}
-		if(phodevi::is_macosx())
+		$halt_screensaver = pts_module::read_variable('HALT_SCREENSAVER');
+		if((!empty($halt_screensaver) && !pts_strings::string_bool($halt_screensaver)) || (phodevi::read_property('system', 'display-server') == null && getenv('XDG_CURRENT_DESKTOP') == false))
+		{
+			return pts_module::MODULE_UNLOAD;
+		}
+		if(phodevi::is_macos())
 		{
 			// Right now there doesn't appear to be a better way to disable OS X screensaver automatically...
 			return pts_module::MODULE_UNLOAD;
@@ -76,7 +81,7 @@ class toggle_screensaver extends pts_module_interface
 
 			$sleep_display_ac = trim(shell_exec(self::$gnome_gconftool . ' -g /apps/gnome-power-manager/timeout/sleep_display_ac 2>&1'));
 
-			if($sleep_display_ac != 0)
+			if(is_numeric($sleep_display_ac) && $sleep_display_ac != 0)
 			{
 				// Don't sleep the display when on AC power
 				shell_exec(self::$gnome_gconftool . ' --type int --set /apps/gnome-power-manager/timeout/sleep_display_ac 0 2>&1');
@@ -109,6 +114,16 @@ class toggle_screensaver extends pts_module_interface
 				self::$gnome3_screensaver_halted = pts_strings::last_in_string($is_gnome3_screensaver_enabled);
 			}
 
+			// GNOME 3.x Lock-Screen
+			$is_gnome3_lockscreen_enabled = trim(shell_exec('gsettings get org.gnome.desktop.lockdown disable-lock-screen 2>&1'));
+
+			if(stripos($is_gnome3_lockscreen_enabled, 'no such key') === false && pts_strings::last_in_string($is_gnome3_lockscreen_enabled) == 'false')
+			{
+				// Stop the GNOME 3.x Lock Screen
+				shell_exec('gsettings set org.gnome.desktop.lockdown disable-lock-screen true 2>&1');
+				self::$gnome3_lockscreen_disabled = true;
+			}
+
 			// This GNOME3 GSettings method is deprecated on distributions like GNOME 3.8 with Fedora 19
 			$is_gnome3_screensaver_enabled_old = trim(shell_exec('gsettings get org.gnome.desktop.screensaver idle-activation-enabled 2>&1'));
 
@@ -189,6 +204,11 @@ class toggle_screensaver extends pts_module_interface
 			// Restore the GNOME Screensaver
 			shell_exec('gsettings set org.gnome.desktop.session idle-delay ' . self::$gnome3_screensaver_halted . ' 2>&1');
 		}
+		if(self::$gnome3_lockscreen_disabled)
+		{
+			// Restore the lock screen
+			shell_exec('gsettings set org.gnome.desktop.lockdown disable-lock-screen false 2>&1');
+		}
 		if(self::$gnome3_screensaver_halted_old == true)
 		{
 			// Restore the GNOME Screensaver
@@ -205,7 +225,7 @@ class toggle_screensaver extends pts_module_interface
 		}
 		if(self::$xset)
 		{
-			shell_exec('xset s default');
+			shell_exec('xset s default 2>&1');
 		}
 		if(getenv('DISPLAY') == false && pts_client::executable_in_path('setterm'))
 		{
