@@ -3,8 +3,8 @@
 /*
 	Phoronix Test Suite
 	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
-	Copyright (C) 2010 - 2014, Phoronix Media
-	Copyright (C) 2010 - 2014, Michael Larabel
+	Copyright (C) 2010 - 2020, Phoronix Media
+	Copyright (C) 2010 - 2020, Michael Larabel
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -22,13 +22,43 @@
 
 class pts_external_dependencies
 {
-	public static function install_dependencies(&$test_profiles)
+	public static function packages_that_provide($file)
+	{
+		$pkg_vendor = self::vendor_identifier('package-list');
+		$provides = false;
+		if($file != null && is_file(pts_exdep_generic_parser::get_external_dependency_path() . 'dependency-handlers/' . $pkg_vendor . '_dependency_handler.php'))
+		{
+			require_once(pts_exdep_generic_parser::get_external_dependency_path() . 'dependency-handlers/' . $pkg_vendor . '_dependency_handler.php');
+			eval("\$provides = {$pkg_vendor}_dependency_handler::what_provides(\$file);");
+		}
+
+		if(empty($provides))
+		{
+			// Fallback to see if it's defined by the XML data
+			$f = array($file => '');
+			$t = false;
+			$x = true;
+			$provides = self::check_dependencies_missing_from_system($f, $t, $x);
+		}
+		return !empty($provides) && is_array($provides) ? $provides : false;
+	}
+	public static function startup_handler()
+	{
+		$pkg_vendor = self::vendor_identifier('package-list');
+		if(is_file(pts_exdep_generic_parser::get_external_dependency_path() . 'dependency-handlers/' . $pkg_vendor . '_dependency_handler.php'))
+		{
+			$startup = null;
+			require_once(pts_exdep_generic_parser::get_external_dependency_path() . 'dependency-handlers/' . $pkg_vendor . '_dependency_handler.php');
+			eval("\$startup = {$pkg_vendor}_dependency_handler::startup_handler();");
+			return $startup;
+		}
+		return false;
+	}
+	public static function install_dependencies(&$test_profiles, $no_prompts = false, $skip_tests_with_missing_dependencies = false, $report_progress = false)
 	{
 		// PTS External Dependencies install on distribution
-
-		if(phodevi::is_windows() || phodevi::is_macosx() || pts_flags::no_external_dependencies())
+		if(pts_client::read_env('NO_EXTERNAL_DEPENDENCIES') != false || pts_client::read_env('SKIP_EXTERNAL_DEPENDENCIES') == 1)
 		{
-			// Windows doesn't use any external dependencies
 			return true;
 		}
 
@@ -38,47 +68,59 @@ class pts_external_dependencies
 		{
 			if(!in_array($test_profile, $tests_to_check) && $test_profile->is_supported())
 			{
-				array_push($tests_to_check, $test_profile);
+				$tests_to_check[] = $test_profile;
 			}
 		}
 
 		// Find all of the POSSIBLE test dependencies
-		$required_test_dependencies = array();
+		$required_external_dependencies = array();
+		$required_system_files = array();
+		if($report_progress)
+		{
+			pts_client::$display->test_install_progress_start('Evaluating External Test Dependencies');
+		}
 		foreach($tests_to_check as &$test_profile)
 		{
-			foreach($test_profile->get_dependencies() as $test_dependency)
+			foreach($test_profile->get_external_dependencies() as $test_dependency)
 			{
 				if(empty($test_dependency))
 				{
 					continue;
 				}
 
-				if(isset($required_test_dependencies[$test_dependency]) == false)
+				if(isset($required_external_dependencies[$test_dependency]) == false)
 				{
-					$required_test_dependencies[$test_dependency] = array();
+					$required_external_dependencies[$test_dependency] = array();
 				}
 
-				array_push($required_test_dependencies[$test_dependency], $test_profile);
+				$required_external_dependencies[$test_dependency][] = $test_profile;
+			}
+			foreach($test_profile->get_system_dependencies() as $test_dependency)
+			{
+				if(empty($test_dependency))
+				{
+					continue;
+				}
+
+				if(isset($required_system_files[$test_dependency]) == false)
+				{
+					$required_system_files[$test_dependency] = array();
+				}
+
+				$required_system_files[$test_dependency][] = $test_profile;
 			}
 		}
 
-		if((pts_c::$test_flags & pts_c::skip_tests_with_missing_dependencies))
+		if($skip_tests_with_missing_dependencies)
 		{
 			// Remove tests that have external dependencies that aren't satisfied and then return
 			$generic_packages_needed = array();
-			$required_test_dependencies_copy = $required_test_dependencies;
-			$dependencies_to_install = self::check_dependencies_missing_from_system($required_test_dependencies_copy, $generic_packages_needed);
-			self::remove_tests_with_missing_dependencies($test_profiles, $generic_packages_needed, $required_test_dependencies);
+			$required_external_dependencies_copy = $required_external_dependencies;
+			$dependencies_to_install = self::check_dependencies_missing_from_system($required_external_dependencies_copy, $generic_packages_needed);
+			self::remove_tests_with_missing_dependencies($test_profiles, $generic_packages_needed, $required_external_dependencies);
 			return true;
 		}
 
-		if(!empty($required_test_dependencies))
-		{
-			// The 'common-dependencies' package is any general non-explicitly-required but nice-to-have packages like mesa-utils for providing glxinfo about the system
-			// So if we're going to be installing external dependencies anyways, might as well try to see the common-dependencies are satisfied
-			$required_test_dependencies['common-dependencies'] = array();
-		}
-
 		// Does the user wish to skip any particular dependencies?
 		if(pts_client::read_env('SKIP_EXTERNAL_DEPENDENCIES'))
 		{
@@ -86,25 +128,56 @@ class pts_external_dependencies
 
 			foreach($dependencies_to_skip as $dependency_name)
 			{
-				if(isset($required_test_dependencies[$dependency_name]))
+				if(isset($required_external_dependencies[$dependency_name]))
 				{
-					unset($required_test_dependencies[$dependency_name]);
+					unset($required_external_dependencies[$dependency_name]);
+				}
+				if(isset($required_system_files[$dependency_name]))
+				{
+					unset($required_system_files[$dependency_name]);
 				}
 			}
 		}
 
 		// Make a copy for use to check at end of process to see if all dependencies were actually found
-		$required_test_dependencies_copy = $required_test_dependencies;
+		$required_external_dependencies_copy = $required_external_dependencies;
 
 		// Find the dependencies that are actually missing from the system
-		$dependencies_to_install = self::check_dependencies_missing_from_system($required_test_dependencies);
+		$skip_warning_on_unmet_deps = false;
+		$generic_packages_needed = array();
+		$dependencies_to_install = self::check_dependencies_missing_from_system($required_external_dependencies, $generic_packages_needed, $skip_warning_on_unmet_deps, true);
+		if($report_progress)
+		{
+			pts_client::$display->test_install_progress_completed();
+		}
 
 		// If it's automated and can't install without root, return true if there are no dependencies to do otherwise false
-		if((pts_c::$test_flags & pts_c::auto_mode) && phodevi::is_root() == false)
+		if($no_prompts && phodevi::is_root() == false)
 		{
 			return count($dependencies_to_install) == 0;
 		}
 
+		if(!empty($dependencies_to_install))
+		{
+			// The 'common-dependencies' package is any general non-explicitly-required but nice-to-have packages like mesa-utils for providing glxinfo about the system
+			// So if we're going to be installing external dependencies anyways, might as well try to see the common-dependencies are satisfied
+			$common_test_dependencies['common-dependencies'] = array();
+			$common_to_install = self::check_dependencies_missing_from_system($common_test_dependencies);
+			if(!empty($common_to_install))
+			{
+				$dependencies_to_install = array_merge($dependencies_to_install, $common_to_install);
+			}
+		}
+
+		$system_dependencies = self::check_for_missing_system_files($required_system_files, $report_progress);
+
+		if(!empty($system_dependencies))
+		{
+			$dependencies_to_install = array_merge($dependencies_to_install, $system_dependencies);
+		}
+
+		$dependencies_to_install = array_unique($dependencies_to_install);
+
 		// Do the actual dependency install process
 		if(count($dependencies_to_install) > 0)
 		{
@@ -112,18 +185,18 @@ class pts_external_dependencies
 		}
 
 		// There were some dependencies not supported on this OS or are missing from the distro's XML file
-		if(count($required_test_dependencies) > 0 && count($dependencies_to_install) == 0)
+		if(count($required_external_dependencies) > 0 && count($dependencies_to_install) == 0 && $skip_warning_on_unmet_deps == false)
 		{
 			$exdep_generic_parser = new pts_exdep_generic_parser();
 			$to_report = array();
 
-			foreach(array_keys($required_test_dependencies) as $dependency)
+			foreach(array_keys($required_external_dependencies) as $dependency)
 			{
 				$dependency_data = $exdep_generic_parser->get_package_data($dependency);
 
 				if($dependency_data['possible_packages'] != null)
 				{
-					array_push($to_report, $dependency_data['title'] . PHP_EOL . 'Possible Package Names: ' . $dependency_data['possible_packages']);
+					$to_report[] = $dependency_data['title'] . PHP_EOL . 'Possible Package Names: ' . $dependency_data['possible_packages'];
 				}
 			}
 
@@ -136,7 +209,7 @@ class pts_external_dependencies
 					pts_client::$display->generic_heading($report);
 				}
 
-				if((pts_c::$test_flags ^ pts_c::batch_mode) && (pts_c::$test_flags ^ pts_c::auto_mode))
+				if(!$no_prompts)
 				{
 					echo 'The above dependencies should be installed before proceeding. Press any key when you\'re ready to continue.';
 					pts_user_io::read_user_input();
@@ -147,11 +220,11 @@ class pts_external_dependencies
 
 
 		// Find the dependencies that are still missing from the system
-		if((pts_c::$test_flags ^ pts_c::batch_mode) && (pts_c::$test_flags ^ pts_c::auto_mode))
+		if(!$no_prompts && !defined('PHOROMATIC_PROCESS') && $skip_warning_on_unmet_deps == false)
 		{
 			$generic_packages_needed = array();
-			$required_test_dependencies = $required_test_dependencies_copy;
-			$dependencies_to_install = self::check_dependencies_missing_from_system($required_test_dependencies_copy, $generic_packages_needed);
+			$required_external_dependencies = $required_external_dependencies_copy;
+			$dependencies_to_install = self::check_dependencies_missing_from_system($required_external_dependencies_copy, $generic_packages_needed);
 
 			if(count($generic_packages_needed) > 0)
 			{
@@ -173,7 +246,7 @@ class pts_external_dependencies
 						break;
 					case 'SKIP_TESTS_WITH_MISSING_DEPS':
 						// Unset the tests that have dependencies still missing
-						self::remove_tests_with_missing_dependencies($test_profiles, $generic_packages_needed, $required_test_dependencies);
+						self::remove_tests_with_missing_dependencies($test_profiles, $generic_packages_needed, $required_external_dependencies);
 						break;
 					case 'REATTEMPT_DEP_INSTALL':
 						self::install_packages_on_system($dependencies_to_install);
@@ -244,13 +317,16 @@ class pts_external_dependencies
 		$dependency_names = self::installed_dependency_names();
 		return self::generic_names_to_titles($dependency_names);
 	}
-	private static function check_dependencies_missing_from_system(&$required_test_dependencies, &$generic_names_of_packages_needed = false)
+	private static function check_dependencies_missing_from_system(&$required_test_dependencies, &$generic_names_of_packages_needed = false, &$skip_warning_on_unmet_deps = false, $report_progress = false)
 	{
 		$generic_dependencies_parser = new pts_exdep_generic_parser();
 		$vendor_dependencies_parser = new pts_exdep_platform_parser(self::vendor_identifier('package-list'));
+		$skip_warning_on_unmet_deps = $vendor_dependencies_parser->skip_warning_on_unmet_dependencies();
 		$kernel_architecture = phodevi::read_property('system', 'kernel-architecture');
 		$needed_os_packages = array();
 
+		$required_test_dep_count = count($required_test_dependencies);
+		$i = 0;
 		foreach($required_test_dependencies as $package => $dependents)
 		{
 			if($vendor_dependencies_parser->is_package($package))
@@ -268,16 +344,20 @@ class pts_external_dependencies
 					$generic_package_data = $generic_dependencies_parser->get_package_data($package);
 					$add_dependency = empty($generic_package_data['file_check']) || self::file_missing_check($generic_package_data['file_check']);
 				}
+				else
+				{
+					$add_dependency = true;
+				}
 
 				if($add_dependency && $arch_compliant && $package_data['os_package'] != null)
 				{
 					if(!in_array($package_data['os_package'], $needed_os_packages))
 					{
-						array_push($needed_os_packages, $package_data['os_package']);
+						$needed_os_packages[] = $package_data['os_package'];
 					}
 					if($generic_names_of_packages_needed !== false && !in_array($package, $generic_names_of_packages_needed))
 					{
-						array_push($generic_names_of_packages_needed, $package);
+						$generic_names_of_packages_needed[] = $package;
 					}
 				}
 				else
@@ -285,6 +365,11 @@ class pts_external_dependencies
 					unset($required_test_dependencies[$package]);
 				}
 			}
+			$i++;
+			if($report_progress)
+			{
+				pts_client::$display->test_install_progress_update(($i / $required_test_dep_count));
+			}
 		}
 
 		if(count($required_test_dependencies) > 0)
@@ -303,6 +388,74 @@ class pts_external_dependencies
 
 		return $needed_os_packages;
 	}
+	private static function check_for_missing_system_files(&$required_system_files, $report_progress = false)
+	{
+		$kernel_architecture = phodevi::read_property('system', 'kernel-architecture');
+		$needed_os_packages = array();
+
+		if($report_progress)
+		{
+			pts_client::$display->test_install_progress_start('Evaluating System Dependencies');
+			$system_file_check_count = count($required_system_files);
+		}
+		$i = 0;
+		foreach(array_keys($required_system_files) as $file)
+		{
+			$present = false;
+			if(is_file($file))
+			{
+				$present = true;
+			}
+			if(strpos($file, '.h') !== false && is_file('/usr/include/' . $file))
+			{
+				$present = true;
+			}
+			else if(strpos($file, '.h') !== false && glob('/usr/include/*-linux-gnu/' . $file) != false)
+			{
+				$present = true;
+			}
+			else if(strpos($file, '.so') !== false && glob('/usr/lib*/' . $file) != false)
+			{
+				$present = true;
+			}
+			else if(strpos($file, '.so') !== false && glob('/usr/lib*/*/' . $file) != false)
+			{
+				$present = true;
+			}
+			else if(pts_client::executable_in_path($file))
+			{
+				$present = true;
+			}
+
+			if(!$present)
+			{
+				$processed_pkgs = self::packages_that_provide($file);
+
+				if(!empty($processed_pkgs))
+				{
+					foreach($processed_pkgs as $pkg)
+					{
+						$needed_os_packages[] = $pkg;
+					}
+				}
+			}
+			$i++;
+			if($report_progress)
+			{
+				pts_client::$display->test_install_progress_update(($i / $system_file_check_count));
+			}
+		}
+		if($report_progress)
+		{
+			pts_client::$display->test_install_progress_completed();
+		}
+
+		return $needed_os_packages;
+	}
+	private static function is_present($file)
+	{
+		return is_file($file) || (strpos($file, '*') != false && glob($file));
+	}
 	private static function file_missing_check($file_arr)
 	{
 		// Checks if file is missing
@@ -322,21 +475,21 @@ class pts_external_dependencies
 			{
 				$file[$i] = trim($file[$i]);
 
-				if(is_file($file[$i]) || is_dir($file[$i]) || is_link($file[$i]))
+				if(is_dir($file[$i]) || self::is_present($file[$i]))
 				{
 					$file_is_there = true;
 				}
-				else if(isset($file[$i][2]) && $file[$i][0] != '/')
+				else if(isset($file[$i][1]) && $file[$i][0] != '/')
 				{
 					// See if it's some relative command/path
 
 					if(substr($file[$i], -2) == '.h' || substr($file[$i], -4) == '.hpp')
 					{
 						// May just be a relative header file to look for...
-						$possible_paths = array('/usr/local/include', '/usr/target/include/', '/usr/include/');
+						$possible_paths = array_merge(array('/usr/local/include/', '/usr/target/include/', '/usr/include/'), pts_file_io::glob('/usr/include/*-linux-gnu/'));
 						foreach($possible_paths as $path)
 						{
-							if(is_file($path . $file[$i]) || is_link($path . $file[$i]))
+							if(self::is_present($path . '/' . $file[$i]))
 							{
 								$file_is_there = true;
 							}
@@ -345,19 +498,19 @@ class pts_external_dependencies
 					else if(strpos($file[$i], '.so') !== false || substr($file[$i], -2) == '.a')
 					{
 						// May just be a relative shared library to look for...
-						$possible_paths = array('/usr/local/lib/', '/usr/lib/', '/usr/lib64/', '/usr/lib/x86_64-linux-gnu/', '/usr/lib/i386-linux-gnu/', '/usr/lib/arm-linux-gnueabihf/');
+						$possible_paths = array_merge(array('/usr/local/lib/', '/usr/lib/', '/usr/lib64/', '/usr/lib/arm-linux-gnueabihf/'), pts_file_io::glob('/usr/lib/*-linux-gnu/'));
 
 						if(getenv('LD_LIBRARY_PATH'))
 						{
 							foreach(explode(':', getenv('LD_LIBRARY_PATH')) as $path)
 							{
-								array_push($possible_paths, $path . '/');
+								$possible_paths[] = $path . '/';
 							}
 						}
 
 						foreach($possible_paths as $path)
 						{
-							if(is_file($path . $file[$i]) || is_link($path . $file[$i]))
+							if(self::is_present($path . '/' . $file[$i]))
 							{
 								$file_is_there = true;
 							}
@@ -366,38 +519,25 @@ class pts_external_dependencies
 					else if(strpos($file[$i], '/') === false)
 					{
 						// May just be a command to look for...
-						$possible_paths = array('/usr/local/bin/', '/usr/bin/');
-
-						if(getenv('PATH'))
+						if(pts_client::executable_in_path($file[$i]))
 						{
-							foreach(explode(':', getenv('PATH')) as $path)
-							{
-								array_push($possible_paths, $path . '/');
-							}
-						}
-
-						foreach($possible_paths as $path)
-						{
-							if(is_file($path . $file[$i]) || is_link($path . $file[$i]))
-							{
-								$file_is_there = true;
-							}
+							$file_is_there = true;
 						}
 					}
 				}
 			}
 			$file_missing = $file_missing || !$file_is_there;
 		}
-
 		return $file_missing;
 	}
 	private static function install_packages_on_system($os_packages_to_install)
 	{
 		// Do the actual installing process of packages using the distribution's package management system
-		$vendor_install_file = PTS_EXDEP_PATH . 'scripts/install-' . self::vendor_identifier('installer') . '-packages.sh';
+		$vendor_install_file = pts_exdep_generic_parser::get_external_dependency_path() . 'scripts/install-' . self::vendor_identifier('installer') . '-packages.sh';
+		$pkg_vendor = self::vendor_identifier('package-list');
 
 		// Rebuild the array index since some OS package XML tags provide multiple package names in a single string
-		$os_packages_to_install = explode(' ', implode(' ', $os_packages_to_install));
+		$os_packages_to_install = array_unique(explode(' ', implode(' ', $os_packages_to_install)));
 
 		if(is_file($vendor_install_file))
 		{
@@ -408,31 +548,36 @@ class pts_external_dependencies
 
 			echo shell_exec('sh ' . $vendor_install_file . ' ' . implode(' ', $os_packages_to_install));
 		}
+		else if(is_file(pts_exdep_generic_parser::get_external_dependency_path() . 'dependency-handlers/' . $pkg_vendor . '_dependency_handler.php'))
+		{
+			$installed = null;
+			require_once(pts_exdep_generic_parser::get_external_dependency_path() . 'dependency-handlers/' . $pkg_vendor . '_dependency_handler.php');
+			eval("\$installed = {$pkg_vendor}_dependency_handler::install_dependencies(\$os_packages_to_install);");
+			return $installed;
+		}
 		else
 		{
-			if(phodevi::is_macosx() == false)
-			{
-				echo 'Distribution install script not found!';
-			}
+			echo 'Distribution install script not found!';
 		}
 	}
-	private static function vendor_identifier($type)
+	public static function vendor_identifier($type)
 	{
 		$os_vendor = phodevi::read_property('system', 'vendor-identifier');
 
 		switch($type)
 		{
 			case 'package-list':
-				$file_check_success = is_file(PTS_EXDEP_PATH . 'xml/' . $os_vendor . '-packages.xml');
+				$file_check_success = is_file(pts_exdep_generic_parser::get_external_dependency_path() . 'xml/' . $os_vendor . '-packages.xml');
 				break;
 			case 'installer':
-				$file_check_success = is_file(PTS_EXDEP_PATH . 'scripts/install-' . $os_vendor . '-packages.sh');
+				$file_check_success = is_file(pts_exdep_generic_parser::get_external_dependency_path() . 'scripts/install-' . $os_vendor . '-packages.sh');
 				break;
 		}
 
 		if($file_check_success == false)
 		{
 			// Check the aliases to figure out the upstream distribution
+			$vend_id = $os_vendor;
 			$os_vendor = false;
 			$exdep_generic_parser = new pts_exdep_generic_parser();
 			foreach($exdep_generic_parser->get_vendors_list() as $this_vendor)
@@ -440,7 +585,7 @@ class pts_external_dependencies
 				$exdep_platform_parser = new pts_exdep_platform_parser($this_vendor);
 				$aliases = $exdep_platform_parser->get_aliases();
 
-				if(in_array($os_vendor, $aliases))
+				if(in_array($vend_id, $aliases))
 				{
 					$os_vendor = $this_vendor;
 					break;
@@ -476,7 +621,7 @@ class pts_external_dependencies
 			if(in_array($package, $names))
 			{
 				$package_data = $generic_dependencies_parser->get_package_data($package);
-				array_push($titles, $package_data['title']);
+				$titles[] = $package_data['title'];
 			}
 		}
 		sort($titles);
