@@ -3,8 +3,8 @@
 /*
 	Phoronix Test Suite
 	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
-	Copyright (C) 2010 - 2014, Phoronix Media
-	Copyright (C) 2010 - 2014, Michael Larabel
+	Copyright (C) 2010 - 2021, Phoronix Media
+	Copyright (C) 2010 - 2021, Michael Larabel
 	phodevi_audio.php: The PTS Device Interface object for audio / sound cards
 
 	This program is free software; you can redistribute it and/or modify
@@ -23,22 +23,17 @@
 
 class phodevi_audio extends phodevi_device_interface
 {
-	public static function read_property($identifier)
+	public static function properties()
 	{
-		switch($identifier)
-		{
-			case 'identifier':
-				$property = new phodevi_device_property('audio_processor_string', phodevi::smart_caching);
-				break;
-		}
-
-		return $property;
+		return array(
+			'identifier' => new phodevi_device_property('audio_processor_string', phodevi::smart_caching)
+		);
 	}
 	public static function audio_processor_string()
 	{
 		$audio = null;
 
-		if(phodevi::is_macosx())
+		if(phodevi::is_macos())
 		{
 			// TODO: implement
 		}
@@ -56,7 +51,21 @@ class phodevi_audio extends phodevi_device_interface
 		}
 		else if(phodevi::is_windows())
 		{
-			// TODO: implement
+			$win_sound = array();
+			$win32_sounddevice = shell_exec('powershell "(Get-WMIObject -Class win32_sounddevice | Select Name)"');
+			if(($x = strpos($win32_sounddevice, '----')) !== false)
+			{
+				$win32_sounddevice = trim(substr($win32_sounddevice, $x + 4));
+				foreach(explode("\n", $win32_sounddevice) as $sd)
+				{
+					if(!empty($sd))
+					{
+						$win_sound[] = $sd;
+					}
+				}
+			}
+			$win_sound = array_unique($win_sound);
+			$audio = implode(' + ', $win_sound);
 		}
 		else if(phodevi::is_linux())
 		{
