@@ -3,8 +3,8 @@
 /*
 	Phoronix Test Suite
 	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
-	Copyright (C) 2008 - 2014, Phoronix Media
-	Copyright (C) 2008 - 2014, Michael Larabel
+	Copyright (C) 2008 - 2021, Phoronix Media
+	Copyright (C) 2008 - 2021, Michael Larabel
 	phodevi_gpu.php: The PTS Device Interface object for the graphics processor
 
 	This program is free software; you can redistribute it and/or modify
@@ -23,49 +23,49 @@
 
 class phodevi_gpu extends phodevi_device_interface
 {
-	public static function read_property($identifier)
+	public static function properties()
 	{
-		switch($identifier)
+		return array(
+			'identifier' => new phodevi_device_property('gpu_string', phodevi::std_caching),
+			'model' => new phodevi_device_property('gpu_model', phodevi::smart_caching),
+			'frequency' => new phodevi_device_property('gpu_frequency_string', phodevi::std_caching),
+			'stock-frequency' => new phodevi_device_property('gpu_stock_frequency', phodevi::std_caching),
+			'memory-capacity' => new phodevi_device_property('gpu_memory_size', phodevi::smart_caching),
+			'oc-offset-string' => new phodevi_device_property('gpu_oc_offset_string', phodevi::no_caching),
+			'aa-level' => new phodevi_device_property('gpu_aa_level', phodevi::no_caching),
+			'af-level' => new phodevi_device_property('gpu_af_level', phodevi::no_caching),
+			'compute-cores' => new phodevi_device_property('gpu_compute_cores', phodevi::smart_caching),
+			'available-modes' => new phodevi_device_property('gpu_available_modes', phodevi::std_caching),
+			'screen-resolution' => new phodevi_device_property('gpu_screen_resolution', phodevi::std_caching),
+			'screen-resolution-string' => new phodevi_device_property('gpu_screen_resolution_string', phodevi::std_caching),
+			'2d-acceleration' => new phodevi_device_property('gpu_2d_acceleration', phodevi::std_caching),
+			'device-id' => new phodevi_device_property('gpu_pci_device_id', phodevi::smart_caching),
+			);
+	}
+	public static function gpu_pci_device_id()
+	{
+		$device_id = null;
+		if(phodevi::is_nvidia_graphics())
 		{
-			case 'identifier':
-				$property = new phodevi_device_property('gpu_string', phodevi::std_caching);
-				break;
-			case 'model':
-				$property = new phodevi_device_property('gpu_model', phodevi::smart_caching);
-				break;
-			case 'frequency':
-				$property = new phodevi_device_property('gpu_frequency_string', phodevi::std_caching);
-				break;
-			case 'stock-frequency':
-				$property = new phodevi_device_property('gpu_stock_frequency', phodevi::std_caching);
-				break;
-			case 'memory-capacity':
-				$property = new phodevi_device_property('gpu_memory_size', phodevi::smart_caching);
-				break;
-			case 'aa-level':
-				$property = new phodevi_device_property('gpu_aa_level', phodevi::no_caching);
-				break;
-			case 'af-level':
-				$property = new phodevi_device_property('gpu_af_level', phodevi::no_caching);
-				break;
-			case 'compute-cores':
-				$property = new phodevi_device_property('gpu_compute_cores', phodevi::smart_caching);
-				break;
-			case 'available-modes':
-				$property = new phodevi_device_property('gpu_available_modes', phodevi::std_caching);
-				break;
-			case 'screen-resolution':
-				$property = new phodevi_device_property('gpu_screen_resolution', phodevi::std_caching);
-				break;
-			case 'screen-resolution-string':
-				$property = new phodevi_device_property('gpu_screen_resolution_string', phodevi::std_caching);
-				break;
-			case '2d-acceleration':
-				$property = new phodevi_device_property('gpu_2d_acceleration', phodevi::std_caching);
-				break;
+			$nvidia_id = explode(',', phodevi_parser::read_nvidia_extension('PCIID'));
+			if(!empty($nvidia_id))
+			{
+				$device_id = array_pop($nvidia_id);
+			}
+		}
+		else if(phodevi::is_linux())
+		{
+			foreach(pts_file_io::glob('/sys/class/drm/card*/device/device') as $device)
+			{
+				$device_id = str_replace('0x', '', pts_file_io::file_get_contents($device));
+				if(!empty($device_id))
+				{
+					break;
+				}
+			}
 		}
 
-		return $property;
+		return $device_id;
 	}
 	public static function gpu_2d_acceleration()
 	{
@@ -115,7 +115,7 @@ class phodevi_gpu extends phodevi_device_interface
 	}
 	public static function gpu_set_resolution($args)
 	{
-		if(count($args) != 2 || phodevi::is_windows() || phodevi::is_macosx() || !pts_client::executable_in_path('xrandr'))
+		if(count($args) != 2 || phodevi::is_windows() || phodevi::is_macos() || !pts_client::executable_in_path('xrandr'))
 		{
 			return false;
 		}
@@ -127,6 +127,22 @@ class phodevi_gpu extends phodevi_device_interface
 
 		return phodevi::read_property('gpu', 'screen-resolution') == array($width, $height); // Check if video resolution set worked
 	}
+	public static function gpu_oc_offset_string()
+	{
+		$offset = 0;
+
+		if(is_file('/sys/class/drm/card0/device/pp_sclk_od'))
+		{
+			// AMDGPU OverDrive
+			$pp_sclk_od = pts_file_io::file_get_contents('/sys/class/drm/card0/device/pp_sclk_od');
+			if(is_numeric($pp_sclk_od) && $pp_sclk_od > 0)
+			{
+				$offset = 'AMD OverDrive GPU Overclock: ' . $pp_sclk_od . '%';
+			}
+		}
+
+		return $offset;
+	}
 	public static function gpu_aa_level()
 	{
 		// Determine AA level if over-rode
@@ -158,77 +174,6 @@ class phodevi_gpu extends phodevi_device_interface
 					break;
 			}
 		}
-		else if(phodevi::is_ati_graphics() && phodevi::is_linux())
-		{
-			$ati_fsaa = phodevi_linux_parser::read_amd_pcsdb('OpenGL,AntiAliasSamples');
-			$ati_fsaa_filter = phodevi_linux_parser::read_amd_pcsdb('OpenGL,AAF');
-
-			if(!empty($ati_fsaa))
-			{
-				if($ati_fsaa_filter == '0x00000000')
-				{
-					// Filter: Box
-					switch($ati_fsaa)
-					{
-						case '0x00000002':
-							$aa_level = '2x Box';
-							break;
-						case '0x00000004':
-							$aa_level = '4x Box';
-							break;
-						case '0x00000008':
-							$aa_level = '8x Box';
-							break;
-					}
-				}
-				else if($ati_fsaa_filter == '0x00000001')
-				{
-					// Filter: Narrow-tent
-					switch($ati_fsaa)
-					{
-						case '0x00000002':
-							$aa_level = '4x Narrow-tent';
-							break;
-						case '0x00000004':
-							$aa_level = '8x Narrow-tent';
-							break;
-						case '0x00000008':
-							$aa_level = '12x Narrow-tent';
-							break;
-					}
-				}
-				else if($ati_fsaa_filter == '0x00000002')
-				{
-					// Filter: Wide-tent
-					switch($ati_fsaa)
-					{
-						case '0x00000002':
-							$aa_level = '6x Wide-tent';
-							break;
-						case '0x00000004':
-							$aa_level = '8x Wide-tent';
-							break;
-						case '0x00000008':
-							$aa_level = '16x Wide-tent';
-							break;
-					}
-
-				}
-				else if($ati_fsaa_filter == '0x00000003')
-				{
-					// Filter: Edge-detect
-					switch($ati_fsaa)
-					{
-						case '0x00000004':
-							$aa_level = '12x Edge-detect';
-							break;
-						case '0x00000008':
-							$aa_level = '24x Edge-detect';
-							break;
-					}
-				}
-			}
-		}
 		else if(phodevi::is_mesa_graphics())
 		{
 			$gallium_msaa = getenv('GALLIUM_MSAA');
@@ -243,7 +188,7 @@ class phodevi_gpu extends phodevi_device_interface
 			$gl_msaa = getenv('__GL_FSAA_MODE');
 			if(is_numeric($gl_msaa) && $gl_msaa > 0)
 			{
-				$aa_level = '__GL_FSAA_MODE=' . $gallium_msaa;
+				$aa_level = '__GL_FSAA_MODE=' . $gl_msaa;
 			}
 		}
 
@@ -274,29 +219,6 @@ class phodevi_gpu extends phodevi_device_interface
 					break;
 			}
 		}
-		else if(phodevi::is_ati_graphics() && phodevi::is_linux())
-		{
-			$ati_af = phodevi_linux_parser::read_amd_pcsdb('OpenGL,AnisoDegree');
-
-			if(!empty($ati_af))
-			{
-				switch($ati_af)
-				{
-					case '0x00000002':
-						$af_level = '2x';
-						break;
-					case '0x00000004':
-						$af_level = '4x';
-						break;
-					case '0x00000008':
-						$af_level = '8x';
-						break;
-					case '0x00000010':
-						$af_level = '16x';
-						break;
-				}
-			}
-		}
 		else if(getenv('__GL_LOG_MAX_ANISO'))
 		{
 			$max_aniso = getenv('__GL_LOG_MAX_ANISO');
@@ -393,7 +315,7 @@ class phodevi_gpu extends phodevi_device_interface
 			}
 		}
 
-		if(phodevi::is_macosx())
+		if(phodevi::is_macos())
 		{
 			$info = pts_strings::trim_explode(' ', phodevi_osx_parser::read_osx_system_profiler('SPDisplaysDataType', 'Resolution'));
 			$resolution = array();
@@ -465,6 +387,24 @@ class phodevi_gpu extends phodevi_device_interface
 					$resolution = $virtual_size;
 				}
 			}
+
+			if($resolution == false && phodevi::is_bsd())
+			{
+				if(($x = strpos(phodevi::$vfs->dmesg, 'VT(efifb): resolution ')) !== false)
+				{
+					$info = substr(phodevi::$vfs->dmesg, $x + 22);
+					$info = trim(substr($info, 0, strpos($info, PHP_EOL)));
+					$res = explode('x', $info);
+					if(count($res) == 2 && is_numeric($res[0]) && is_numeric($res[1]))
+					{
+						$resolution = $res;
+					}
+				}
+			}
+		}
+		else if(phodevi::is_windows())
+		{
+			$resolution = array(phodevi_windows_parser::get_wmi_object('Win32_VideoController', 'CurrentHorizontalResolution'), phodevi_windows_parser::get_wmi_object('Win32_VideoController', 'CurrentVerticalResolution'));
 		}
 
 		return $resolution == false ? array(-1, -1) : $resolution;
@@ -487,7 +427,6 @@ class phodevi_gpu extends phodevi_device_interface
 		$current_resolution = phodevi::read_property('gpu', 'screen-resolution');
 		$current_pixel_count = $current_resolution[0] * $current_resolution[1];
 		$available_modes = array();
-		$supported_ratios = array(1.60, 1.25, 1.33, 1.70, 1.71, 1.78);
 		$ignore_modes = array(
 			array(640, 400),
 			array(720, 480), array(832, 624),
@@ -511,7 +450,7 @@ class phodevi_gpu extends phodevi_device_interface
 		}
 
 		// Attempt reading available modes from xrandr
-		if(pts_client::executable_in_path('xrandr') && !phodevi::is_macosx()) // MacOSX has xrandr but currently on at least my setup will emit a Bus Error when called
+		if(pts_client::executable_in_path('xrandr') && !phodevi::is_macos()) // MacOSX has xrandr but currently on at least my setup will emit a Bus Error when called
 		{
 			$xrandr_lines = array_reverse(explode("\n", shell_exec('xrandr 2>&1')));
 
@@ -542,22 +481,18 @@ class phodevi_gpu extends phodevi_device_interface
 			}
 		}
 
-		if(count($available_modes) <= 2)
+		if(true) // XXX can remove this if it turns out to be fine
 		{
 			// Fallback to providing stock modes
 			$stock_modes = array(
 				array(800, 600),
 				array(1024, 768),
-				array(1280, 800),
 				array(1280, 1024),
-				array(1400, 900),
-				array(1400, 1050),
-				array(1600, 900),
-				array(1680, 1050),
 				array(1600, 1200),
 				array(1920, 1080),
 				array(1920, 1200),
-				array(2560, 1600));
+				array(2560, 1440),
+				array(3840, 2160));
 			$available_modes = array();
 
 			for($i = 0; $i < count($stock_modes); $i++)
@@ -588,11 +523,6 @@ class phodevi_gpu extends phodevi_device_interface
 				// For displays larger than 1024 x 600, drop modes below 800 x 600 unless llvmpipe is being used
 				unset($available_modes[$mode_index]);
 			}
-			else if($current_pixel_count > 480000 && !in_array($this_ratio, $supported_ratios))
-			{
-				// For displays larger than 800 x 600, ensure reading from a supported ratio
-				unset($available_modes[$mode_index]);
-			}
 			else if(in_array($mode, $ignore_modes))
 			{
 				// Mode is to be ignored
@@ -675,18 +605,54 @@ class phodevi_gpu extends phodevi_device_interface
 		{
 			$video_ram = $NVIDIA / 1024;
 		}
-		else if(phodevi::is_macosx())
+		else if(($nvidia_smi = pts_client::executable_in_path('nvidia-smi')))
+		{
+			$smi_output = shell_exec(escapeshellarg($nvidia_smi) . ' -q -d MEMORY');
+			$mem = strpos($smi_output, 'Total');
+			if($mem !== false)
+			{
+				$mem = substr($smi_output, strpos($smi_output, ':', $mem) + 1);
+				$mem = trim(substr($mem, 0, strpos($mem, 'MiB')));
+
+				if(is_numeric($mem) && $mem > 0)
+				{
+					$video_ram = $mem;
+				}
+			}
+		}
+		else if(phodevi::is_macos())
 		{
 			$info = phodevi_osx_parser::read_osx_system_profiler('SPDisplaysDataType', 'VRAM');
 			$info = explode(' ', $info);
 			$video_ram = $info[0];
 
-			if($info[1] == 'GB')
+			if(isset($info[1]) && $info[1] == 'GB')
 			{
 				$video_ram *= 1024;
 			}
 		}
+		else if(phodevi::is_windows())
+		{
+			$video_ram = phodevi_windows_parser::get_wmi_object('Win32_VideoController', 'AdapterRAM');
+			if(is_numeric($video_ram) && $video_ram > 1048576)
+			{
+				$video_ram = $video_ram / 1048576;
+			}
+		}
+		else
+		{
+			// Try reading video memoty from GLX_MESA_query_renderer output in glxinfo
+			$glxinfo_video_mem = str_replace('MB', null, phodevi_parser::glxinfo_read_line('Video memory'));
 
+			if(is_numeric($glxinfo_video_mem) && $glxinfo_video_mem > 1)
+			{
+				// Do some rounding as at least AMDGPU/RadeonSI reports less than real amount
+				$video_ram = $glxinfo_video_mem % 128 === 0 ? $glxinfo_video_mem : round(($glxinfo_video_mem + 128 / 2) / 128) * 128;
+			}
+		}
+
+		// Disabling 27 Nov 2016 as seems to cause problems with AMDGPU-0PRO but falls back gracefully outside of this xorg-log parsing for correct amount
+		/*
 		if($video_ram == -1 && isset(phodevi::$vfs->xorg_log))
 		{
 			// Attempt Video RAM detection using X log
@@ -710,6 +676,8 @@ class phodevi_gpu extends phodevi_device_interface
 				}
 			}
 		}
+		*/
+
 		if($video_ram == -1 && isset(phodevi::$vfs->dmesg))
 		{
 			// Fallback to try to find vRAM from dmesg
@@ -738,6 +706,11 @@ class phodevi_gpu extends phodevi_device_interface
 				// Nouveau DRM around Linux ~3.6 reports e.g.: DRM] VRAM: 512 MiB
 				$info = substr($info, ($x + 6));
 				$info = substr($info, 0, strpos($info, ' '));
+
+				if(substr($info, -1) == 'M')
+				{
+					$info = substr($info, 0, -1);
+				}
 			}
 
 			if(is_numeric($info))
@@ -763,7 +736,14 @@ class phodevi_gpu extends phodevi_device_interface
 		}
 		else
 		{
-			return phodevi::read_property('gpu', 'model') . ' ' . phodevi::read_property('gpu', 'frequency');
+			$model = phodevi::read_property('gpu', 'model');
+			$freq_string = null;
+			if(stripos($model, 'NVIDIA') === false)
+			{
+				// 2021: NVIDIA dynamic frequency handling is too sporadic, wish there was a way to query peak freq
+				$freq_string = phodevi::read_property('gpu', 'frequency');
+			}
+			return $model . ($freq_string != null ? ' (' . $freq_string . ')' : null);
 		}
 	}
 	public static function gpu_frequency_string()
@@ -783,7 +763,7 @@ class phodevi_gpu extends phodevi_device_interface
 			$freq_string .= 'MHz';
 		}
 
-		return ($freq_string != null ? ' (' . $freq_string . ')' : null);
+		return $freq_string;
 	}
 	public static function gpu_stock_frequency()
 	{
@@ -791,7 +771,7 @@ class phodevi_gpu extends phodevi_device_interface
 		$core_freq = 0;
 		$mem_freq = 0;
 
-		if(phodevi::is_nvidia_graphics() && phodevi::is_macosx() == false) // NVIDIA GPU
+		if(phodevi::is_nvidia_graphics() && phodevi::is_macos() == false) // NVIDIA GPU
 		{
 			// GPUDefault3DClockFreqs is the default and does not show under/over-clocking
 			$clock_freqs_3d = pts_strings::comma_explode(phodevi_parser::read_nvidia_extension('GPU3DClockFreqs'));
@@ -807,311 +787,287 @@ class phodevi_gpu extends phodevi_device_interface
 				$mem_freq = max($mem_freq, $clock_freqs_current[1]);
 			}
 		}
-		else if(phodevi::is_ati_graphics() && phodevi::is_linux()) // ATI GPU
+		else if(phodevi::is_linux()) // More liberally attempt open-source freq detection than phodevi::is_mesa_graphics()
 		{
-			$od_clocks = phodevi_linux_parser::read_ati_overdrive('CurrentPeak');
-
-			if(is_array($od_clocks) && count($od_clocks) >= 2) // ATI OverDrive
+			if(is_file('/sys/class/drm/card0/device/performance_level'))
 			{
-				list($core_freq, $mem_freq) = $od_clocks;
+				// NOUVEAU
+				/*
+					EXAMPLE OUTPUTS:
+					memory 1000MHz core 500MHz voltage 1300mV fanspeed 100%
+					3: memory 333MHz core 500MHz shader 1250MHz fanspeed 100%
+					c: memory 333MHz core 500MHz shader 1250MHz
+				*/
+
+				$performance_level = pts_file_io::file_get_contents('/sys/class/drm/card0/device/performance_level');
+				$performance_level = explode(' ', $performance_level);
+
+				$core_string = array_search('core', $performance_level);
+				if($core_string !== false && isset($performance_level[($core_string + 1)]))
+				{
+					$core_string = str_ireplace('MHz', null, $performance_level[($core_string + 1)]);
+					if(is_numeric($core_string))
+					{
+						$core_freq = $core_string;
+					}
+				}
+				$mem_string = array_search('memory', $performance_level);
+				if($mem_string !== false && isset($performance_level[($mem_string + 1)]))
+				{
+					$mem_string = str_ireplace('MHz', null, $performance_level[($mem_string + 1)]);
+					if(is_numeric($mem_string))
+					{
+						$mem_freq = $mem_string;
+					}
+				}
+			}
+			else if(is_file('/sys/class/drm/card0/device/pstate'))
+			{
+				// Nouveau
+				// pstate is present with Linux 3.13 as the new performance states on Fermi/Kepler
+				$performance_state = pts_file_io::file_get_contents('/sys/class/drm/card0/device/pstate');
+				$performance_level = substr($performance_state, 0, strpos($performance_state, ' *'));
+				if($performance_level == null)
+				{
+					// Method for Linux 3.17+
+					$performance_level = substr($performance_state, strpos($performance_state, 'AC: ') + 4);
+					if(($t = strpos($performance_level, PHP_EOL)))
+					{
+						$performance_level = substr($performance_level, 0, $t);
+					}
+				}
+				else
+				{
+					// Method for Linux ~3.13 through Linux 3.16
+					$performance_level = substr($performance_level, strrpos($performance_level, ': ') + 2);
+				}
+
+				$performance_level = explode(' ', $performance_level);
+				$core_string = array_search('core', $performance_level);
+				if($core_string !== false && isset($performance_level[($core_string + 1)]))
+				{
+					$core_string = str_ireplace('MHz', null, $performance_level[($core_string + 1)]);
+					if(strpos($core_string, '-') !== false)
+					{
+						// to work around a range of values, e.g.
+						// 0a: core 405-1032 MHz memory 1620 MHz AC DC *
+						$core_string = max(explode('-', $core_string));
+					}
+					if(is_numeric($core_string))
+					{
+						$core_freq = $core_string;
+					}
+				}
+				$mem_string = array_search('memory', $performance_level);
+				if($mem_string !== false && isset($performance_level[($mem_string + 1)]))
+				{
+					$mem_string = str_ireplace('MHz', null, $performance_level[($mem_string + 1)]);
+					if(strpos($mem_string, '-') !== false)
+					{
+						// to work around a range of values, e.g.
+						// 0a: core 405-1032 MHz memory 1620 MHz AC DC *
+						$mem_string = max(explode('-', $mem_string));
+					}
+					if(is_numeric($mem_string))
+					{
+						$mem_freq = $mem_string;
+					}
+				}
+
+			}
+
+			//
+			// RADEON / AMDGPU Logic
+			//
+
+			if(isset(phodevi::$vfs->radeon_pm_info))
+			{
+				// radeon_pm_info should be present with Linux 2.6.34+ but was changed with Linux 3.11 Radeon DPM
+				if(stripos(phodevi::$vfs->radeon_pm_info, 'default'))
+				{
+					foreach(pts_strings::trim_explode("\n", phodevi::$vfs->radeon_pm_info) as $pm_line)
+					{
+						if($pm_line == null)
+						{
+							continue;
+						}
+						list($descriptor, $value) = pts_strings::colon_explode($pm_line);
+
+						switch($descriptor)
+						{
+							case 'default engine clock':
+								$core_freq = pts_arrays::first_element(explode(' ', $value)) / 1000;
+								break;
+							case 'default memory clock':
+								$mem_freq = pts_arrays::first_element(explode(' ', $value)) / 1000;
+								break;
+						}
+					}
+				}
+				if($core_freq == 0 && ($x = stripos(phodevi::$vfs->radeon_pm_info, 'sclk: ')) != false)
+				{
+					$x = substr(phodevi::$vfs->radeon_pm_info, $x + strlen('sclk: '));
+					$x = substr($x, 0, strpos($x, ' '));
+					if(is_numeric($x) && $x > 100)
+					{
+						if($x > 10000)
+						{
+							$x = $x / 100;
+						}
+						$core_freq = $x;
+					}
+					if(($x = stripos(phodevi::$vfs->radeon_pm_info, 'mclk: ')) != false)
+					{
+						$x = substr(phodevi::$vfs->radeon_pm_info, $x + strlen('mclk: '));
+						$x = substr($x, 0, strpos($x, ' '));
+						if(is_numeric($x) && $x > 100)
+						{
+							if($x > 10000)
+							{
+								$x = $x / 100;
+							}
+							$mem_freq = $x;
+						}
+					}
+				}
+			}
+			if($core_freq == null && is_file('/sys/class/drm/card0/device/pp_dpm_sclk'))
+			{
+				$pp = trim(file_get_contents('/sys/class/drm/card0/device/pp_dpm_sclk'));
+				$pp = explode("\n", $pp);
+				$pp = array_pop($pp);
+				if(($x = strpos($pp, ': ')) !== false)
+				{
+					$pp = substr($pp, $x + 2);
+				}
+				$pp = trim(str_replace(array('*', 'Mhz'), '', $pp));
+				if(is_numeric($pp))
+				{
+					$core_freq = $pp;
+
+					if(is_file('/sys/class/drm/card0/device/pp_dpm_mclk'))
+					{
+						$pp = trim(file_get_contents('/sys/class/drm/card0/device/pp_dpm_mclk'));
+						$pp = explode("\n", $pp);
+						$pp = array_pop($pp);
+						if(($x = strpos($pp, ': ')) !== false)
+						{
+							$pp = substr($pp, $x + 2);
+						}
+						$pp = trim(str_replace(array('*', 'Mhz'), '', $pp));
+						if(is_numeric($pp))
+						{
+							$mem_freq = $pp;
+						}
+					}
+				}
+			}
+			if($core_freq == null && isset(phodevi::$vfs->dmesg) && strrpos(phodevi::$vfs->dmesg, ' sclk:'))
+			{
+				// Attempt to read the LAST power level reported to dmesg, this is the current way for Radeon DPM on Linux 3.11+
+				$dmesg_parse = phodevi::$vfs->dmesg;
+				if(($x = strrpos($dmesg_parse, ' sclk:')))
+				{
+					$dmesg_parse = substr($dmesg_parse, $x);
+					$dmesg_parse = explode(' ', substr($dmesg_parse, 0, strpos($dmesg_parse, PHP_EOL)));
+
+					$sclk = array_search('sclk:', $dmesg_parse);
+					if($sclk !== false && isset($dmesg_parse[($sclk + 1)]) && is_numeric($dmesg_parse[($sclk + 1)]))
+					{
+						$sclk = $dmesg_parse[($sclk + 1)];
+						if($sclk > 10000)
+						{
+							$sclk = $sclk / 100;
+						}
+						$core_freq = $sclk;
+					}
+					$mclk = array_search('mclk:', $dmesg_parse);
+					if($mclk !== false && isset($dmesg_parse[($mclk + 1)]) && is_numeric($dmesg_parse[($mclk + 1)]))
+					{
+						$mclk = $dmesg_parse[($mclk + 1)];
+						if($mclk > 10000)
+						{
+							$mclk = $mclk / 100;
+						}
+						$mem_freq = $mclk;
+					}
+				}
+			}
+
+			//
+			// INTEL
+			//
+
+			// try to read the maximum dynamic frequency
+			if($core_freq == 0 && is_file('/sys/class/drm/card0/gt_max_freq_mhz'))
+			{
+				$gt_max_freq_mhz = pts_file_io::file_get_contents('/sys/class/drm/card0/gt_max_freq_mhz');
+				if(is_numeric($gt_max_freq_mhz) && $gt_max_freq_mhz > 100)
+				{
+					// Tested on Linux 3.11. Assume the max frequency on any competent GPU is beyond 100MHz
+					$core_freq = $gt_max_freq_mhz;
+				}
+			}
+			if($core_freq == 0 && is_file('/sys/kernel/debug/dri/0/i915_max_freq'))
+			{
+				$i915_max_freq = pts_file_io::file_get_contents('/sys/kernel/debug/dri/0/i915_max_freq');
+				$freq_mhz = substr($i915_max_freq, strpos($i915_max_freq, ': ') + 2);
+				if(is_numeric($freq_mhz))
+				{
+					$core_freq = $freq_mhz;
+				}
+			}
+			// Fallback to base frequency
+			if($core_freq == 0 && isset(phodevi::$vfs->i915_cur_delayinfo))
+			{
+				$i915_cur_delayinfo = phodevi::$vfs->i915_cur_delayinfo;
+				$freq = strpos($i915_cur_delayinfo, 'Max overclocked frequency: ');
+
+				if($freq === false)
+				{
+					$freq = strpos($i915_cur_delayinfo, 'Max non-overclocked (RP0) frequency: ');
+				}
+				if($freq === false)
+				{
+					$freq = strpos($i915_cur_delayinfo, 'Nominal (RP1) frequency: ');
+				}
+
+				if($freq !== false)
+				{
+					$freq_mhz = substr($i915_cur_delayinfo, strpos($i915_cur_delayinfo, ': ', $freq) + 2);
+					$freq_mhz = trim(substr($freq_mhz, 0, strpos($freq_mhz, 'MHz')));
+					if(is_numeric($freq_mhz))
+					{
+						$core_freq = $freq_mhz;
+					}
+				}
 			}
 		}
-		else if(phodevi::is_mesa_graphics())
+		else if(($nvidia_smi = pts_client::executable_in_path('nvidia-smi')))
 		{
-			switch(phodevi::read_property('system', 'display-driver'))
+			$smi_output = shell_exec(escapeshellarg($nvidia_smi) . ' -q -d CLOCK');
+			$mem = strpos($smi_output, 'Max Clocks');
+			if($mem !== false)
 			{
-				case 'nouveau':
-					if(is_file('/sys/class/drm/card0/device/performance_level'))
-					{
-						/*
-							EXAMPLE OUTPUTS:
-							memory 1000MHz core 500MHz voltage 1300mV fanspeed 100%
-							3: memory 333MHz core 500MHz shader 1250MHz fanspeed 100%
-							c: memory 333MHz core 500MHz shader 1250MHz
-						*/
-
-						$performance_level = pts_file_io::file_get_contents('/sys/class/drm/card0/device/performance_level');
-						$performance_level = explode(' ', $performance_level);
-
-						$core_string = array_search('core', $performance_level);
-						if($core_string !== false && isset($performance_level[($core_string + 1)]))
-						{
-							$core_string = str_ireplace('MHz', null, $performance_level[($core_string + 1)]);
-							if(is_numeric($core_string))
-							{
-								$core_freq = $core_string;
-							}
-						}
-
-						$mem_string = array_search('memory', $performance_level);
-						if($mem_string !== false && isset($performance_level[($mem_string + 1)]))
-						{
-							$mem_string = str_ireplace('MHz', null, $performance_level[($mem_string + 1)]);
-							if(is_numeric($mem_string))
-							{
-								$mem_freq = $mem_string;
-							}
-						}
-					}
-					else if(is_file('/sys/class/drm/card0/device/pstate'))
-					{
-						// pstate is present with Linux 3.13 as the new performance states on Fermi/Kepler
-						$performance_state = pts_file_io::file_get_contents('/sys/class/drm/card0/device/pstate');
-						$performance_level = substr($performance_state, 0, strpos($performance_state, ' *'));
-						$performance_level = substr($performance_level, strrpos($performance_level, ': ') + 2);
-						$performance_level = explode(' ', $performance_level);
-
-						$core_string = array_search('core', $performance_level);
-						if($core_string !== false && isset($performance_level[($core_string + 1)]))
-						{
-							$core_string = str_ireplace('MHz', null, $performance_level[($core_string + 1)]);
-							if(is_numeric($core_string))
-							{
-								$core_freq = $core_string;
-							}
-						}
-
-						$mem_string = array_search('memory', $performance_level);
-						if($mem_string !== false && isset($performance_level[($mem_string + 1)]))
-						{
-							$mem_string = str_ireplace('MHz', null, $performance_level[($mem_string + 1)]);
-							if(is_numeric($mem_string))
-							{
-								$mem_freq = $mem_string;
-							}
-						}
-
-					}
-					break;
-				case 'radeon':
-					if(isset(phodevi::$vfs->radeon_pm_info))
-					{
-						// radeon_pm_info should be present with Linux 2.6.34+ but was changed with Linux 3.11 Radeon DPM
-						if(stripos(phodevi::$vfs->radeon_pm_info, 'default'))
-						{
-							foreach(pts_strings::trim_explode("\n", phodevi::$vfs->radeon_pm_info) as $pm_line)
-							{
-								if($pm_line == null)
-								{
-									continue;
-								}
-
-								list($descriptor, $value) = pts_strings::colon_explode($pm_line);
-
-								switch($descriptor)
-								{
-									case 'default engine clock':
-										$core_freq = pts_arrays::first_element(explode(' ', $value)) / 1000;
-										break;
-									case 'default memory clock':
-										$mem_freq = pts_arrays::first_element(explode(' ', $value)) / 1000;
-										break;
-								}
-							}
-						}
-						if($core_freq == 0 && ($x = stripos(phodevi::$vfs->radeon_pm_info, 'sclk: ')) != false)
-						{
-							$x = substr(phodevi::$vfs->radeon_pm_info, $x + strlen('sclk: '));
-							$x = substr($x, 0, strpos($x, ' '));
-							if(is_numeric($x) && $x > 100)
-							{
-								if($x > 10000)
-								{
-									$x = $x / 100;
-								}
-								$core_freq = $x;
-							}
-
-							if(($x = stripos(phodevi::$vfs->radeon_pm_info, 'mclk: ')) != false)
-							{
-								$x = substr(phodevi::$vfs->radeon_pm_info, $x + strlen('mclk: '));
-								$x = substr($x, 0, strpos($x, ' '));
-								if(is_numeric($x) && $x > 100)
-								{
-									if($x > 10000)
-									{
-										$x = $x / 100;
-									}
-									$mem_freq = $x;
-								}
-							}
-						}
-					}
-
-					if($core_freq == null)
-					{
-						// Attempt to read the LAST power level reported to dmesg, this is the current way for Radeon DPM on Linux 3.11+
-						$dmesg_parse = isset(phodevi::$vfs->dmesg) ? phodevi::$vfs->dmesg : null;
-						if(($x = strrpos($dmesg_parse, ' sclk:')))
-						{
-							$dmesg_parse = substr($dmesg_parse, $x);
-							$dmesg_parse = explode(' ', substr($dmesg_parse, 0, strpos($dmesg_parse, PHP_EOL)));
-
-							$sclk = array_search('sclk:', $dmesg_parse);
-							if($sclk !== false && isset($dmesg_parse[($sclk + 1)]) && is_numeric($dmesg_parse[($sclk + 1)]))
-							{
-								$sclk = $dmesg_parse[($sclk + 1)];
-								if($sclk > 10000)
-								{
-									$sclk = $sclk / 100;
-								}
-
-								$core_freq = $sclk;
-							}
-							$mclk = array_search('mclk:', $dmesg_parse);
-							if($mclk !== false && isset($dmesg_parse[($mclk + 1)]) && is_numeric($dmesg_parse[($mclk + 1)]))
-							{
-								$mclk = $dmesg_parse[($mclk + 1)];
-								if($mclk > 10000)
-								{
-									$mclk = $mclk / 100;
-								}
-
-								$mem_freq = $mclk;
-							}
-						}
-					}
-
-					if($core_freq == null)
-					{
-						// Old ugly way of handling the clock information
-						$log_parse = isset(phodevi::$vfs->xorg_log) ? phodevi::$vfs->xorg_log : null;
-						if(($engine_clock = strpos($log_parse, 'Default Engine Clock: ')))
-						{
-							$core_freq = substr($log_parse, $engine_clock + 22);
-							$core_freq = substr($core_freq, 0, strpos($core_freq, "\n"));
-							$core_freq = is_numeric($core_freq) ? $core_freq / 1000 : 0;
-
-							if($core_freq && ($mem_clock = strpos($log_parse, 'Default Memory Clock: ')))
-							{
-								$mem_freq = substr($log_parse, $mem_clock + 22);
-								$mem_freq = substr($mem_freq, 0, strpos($mem_freq, "\n"));
-								$mem_freq = is_numeric($mem_freq) ? $mem_freq / 1000 : 0;
-							}
-							else
-							{
-								$core_freq = 0;
-							}
-						}
-					}
-					break;
-				case 'intel':
-					// try to read the maximum dynamic frequency
-					if(is_file('/sys/class/drm/card0/gt_max_freq_mhz'))
-					{
-						$gt_max_freq_mhz = pts_file_io::file_get_contents('/sys/class/drm/card0/gt_max_freq_mhz');
-
-						if(is_numeric($gt_max_freq_mhz) && $gt_max_freq_mhz > 100)
-						{
-							// Tested on Linux 3.11. Assume the max frequency on any competent GPU is beyond 100MHz
-							$core_freq = $gt_max_freq_mhz;
-						}
-					}
-
-					if($core_freq == 0 && is_file('/sys/kernel/debug/dri/0/i915_max_freq'))
-					{
-						$i915_max_freq = pts_file_io::file_get_contents('/sys/kernel/debug/dri/0/i915_max_freq');
-						$freq_mhz = substr($i915_max_freq, strpos($i915_max_freq, ': ') + 2);
-
-						if(is_numeric($freq_mhz))
-						{
-							$core_freq = $freq_mhz;
-						}
-					}
-
-					// Fallback to base frequency
-					if($core_freq == 0 && isset(phodevi::$vfs->i915_cur_delayinfo))
-					{
-						$i915_cur_delayinfo = phodevi::$vfs->i915_cur_delayinfo;
-						$freq = strpos($i915_cur_delayinfo, 'Max overclocked frequency: ');
-
-						if($freq === false)
-						{
-							$freq = strpos($i915_cur_delayinfo, 'Max non-overclocked (RP0) frequency: ');
-						}
-
-						if($freq === false)
-						{
-							$freq = strpos($i915_cur_delayinfo, 'Nominal (RP1) frequency: ');
-						}
-
-						if($freq !== false)
-						{
-							$freq_mhz = substr($i915_cur_delayinfo, strpos($i915_cur_delayinfo, ': ', $freq) + 2);
-							$freq_mhz = trim(substr($freq_mhz, 0, strpos($freq_mhz, 'MHz')));
-
-							if(is_numeric($freq_mhz))
-							{
-								$core_freq = $freq_mhz;
-							}
-						}
-					}
-					break;
+				$core_clock = substr($smi_output, stripos($smi_output, 'Graphics', $mem) + 9);
+				$core_freq = trim(str_replace(':', '', substr($core_clock, 0, strpos($core_clock, 'MHz'))));
+				$mem_clock = substr($smi_output, stripos($smi_output, 'Memory', $mem) + 8);
+				$mem_freq = trim(str_replace(':', '', substr($mem_clock, 0, strpos($mem_clock, 'MHz'))));
 			}
 		}
 
-		if(!is_numeric($core_freq))
-		{
-			$core_freq = 0;
-		}
-		if(!is_numeric($mem_freq))
-		{
-			$mem_freq = 0;
-		}
+		$core_freq = !is_numeric($core_freq) ? 0 : round($core_freq);
+		$mem_freq = !is_numeric($mem_freq) ? 0 : round($mem_freq);
 
 		return array($core_freq, $mem_freq);
 	}
 	public static function gpu_model()
 	{
 		// Report graphics processor string
-		$info = phodevi_parser::read_glx_renderer();
+		$info = str_replace('(R)', '', phodevi_parser::read_glx_renderer());
 		$video_ram = phodevi::read_property('gpu', 'memory-capacity');
 
-		if(phodevi::is_ati_graphics() && phodevi::is_linux())
-		{
-			$crossfire_status = phodevi_linux_parser::read_amd_pcsdb('SYSTEM/Crossfire/chain/*,Enable');
-			$crossfire_status = pts_arrays::to_array($crossfire_status);
-			$crossfire_card_count = 0;
-
-			for($i = 0; $i < count($crossfire_status); $i++)
-			{
-				if($crossfire_status[$i] == '0x00000001')
-				{
-					$crossfire_card_count += 2; // For now assume each chain is 2 cards, but proper way would be NumSlaves + 1
-				}
-			}			
-
-			$adapters = phodevi_linux_parser::read_amd_graphics_adapters();
-
-			if(count($adapters) > 0)
-			{
-				$video_ram = $video_ram > 64 ? ' ' . $video_ram . 'MB' : null; // assume more than 64MB of vRAM
-
-				if($crossfire_card_count > 1 && $crossfire_card_count <= count($adapters))
-				{
-					$unique_adapters = array_unique($adapters);
-
-					if(count($unique_adapters) == 1)
-					{
-						if(strpos($adapters[0], 'X2') > 0 && $crossfire_card_count > 1)
-						{
-							$crossfire_card_count -= 1;
-						}
-
-						$info = $crossfire_card_count . ' x ' . $adapters[0] . $video_ram . ' CrossFire';
-					}
-					else
-					{
-						$info = implode(', ', $unique_adapters) . ' CrossFire';
-					}
-				}
-				else
-				{
-					$info = $adapters[0] . $video_ram;
-				}
-			}
-		}
-		else if(phodevi::is_macosx())
+		if(phodevi::is_macos())
 		{
 			$system_profiler_info = implode(' + ', phodevi_osx_parser::read_osx_system_profiler('SPDisplaysDataType', 'ChipsetModel', true));
 
@@ -1212,19 +1168,50 @@ class phodevi_gpu extends phodevi_device_interface
 					$info = substr($xorg_log, strrpos($xorg_log, 'Found ') + 6);
 				}
 			}
+
+			if($info == null)
+			{
+				$info = phodevi_bsd_parser::read_pciconf_by_class('display');
+
+
+				if(strlen($info) > 60)
+				{
+					$info = null;
+				}
+			}
 		}
 		else if(phodevi::is_windows())
 		{
-			$info = phodevi_windows_parser::read_cpuz('Display Adapters', 'Name');
+			$windows_gpu = phodevi_windows_parser::get_wmi_object_multi('Win32_VideoController', 'Name');
+			if(count($windows_gpu) > 1 && ($x = array_search('Microsoft Basic Display Adapter', $windows_gpu)) !== false)
+			{
+				unset($windows_gpu[$x]);
+			}
+			$info = str_replace('(TM)', null, implode(' + ', $windows_gpu));
 		}
-	
-		if(empty($info) || strpos($info, 'Mesa ') !== false || strpos($info, 'Gallium ') !== false)
+
+		if(empty($info) || strpos($info, 'Mesa ') !== false || strpos($info, 'Gallium ') !== false || strpos($info, 'DRM ') !== false)
 		{
-			if(phodevi::is_windows() == false)
+			if(($x = strpos($info, ' on ')) !== false)
+			{
+				// to remove section like "Gallium 0.4 on AMD POLARIS"
+				$info = substr($info, $x + 4);
+			}
+			if(strpos($info, 'Intel ') !== false)
+			{
+				// Intel usually has e.g. TGL GT2 or other info within
+				$info = str_replace(array('(', ')'), null, $info);
+			}
+			if(($x = strpos($info, ' (')) !== false)
+			{
+				$info = substr($info, 0, $x);
+			}
+
+			if(phodevi::is_windows() == false && strpos($info, 'Intel ') === false)
 			{
 				$info_pci = phodevi_linux_parser::read_pci('VGA compatible controller', false);
 
-				if(!empty($info_pci))
+				if(!empty($info_pci) && strpos($info_pci, 'Device ') === false)
 				{
 					$info = $info_pci;
 
@@ -1354,6 +1341,37 @@ class phodevi_gpu extends phodevi_device_interface
 			}
 		}
 
+		if(empty($info) && (($nvidia_smi = pts_client::executable_in_path('nvidia-smi')) || ($nvidia_smi = pts_client::executable_in_path('nvidia-smi.exe'))))
+		{
+			// This works for some headless configurations or with Windows WSL2
+			$nvidia_smi = shell_exec($nvidia_smi . ' -L 2>&1');
+			if(($x = strpos($nvidia_smi, 'GPU 0: ')) !== false)
+			{
+				$nvidia_smi = substr($nvidia_smi, $x + 7);
+
+				if(($x = strpos($nvidia_smi, PHP_EOL)) !== false)
+				{
+					$nvidia_smi = substr($nvidia_smi, 0, $x);
+				}
+
+				$info = trim($nvidia_smi);
+			}
+		}
+
+		if(($x = strpos($info, ' (')) !== false)
+		{
+			$info = substr($info, 0, $x);
+		}
+
+		if(empty($info) && isset(phodevi::$vfs->xorg_log))
+		{
+			// ARM Mali DDX driver detection fallback
+			if(strpos(phodevi::$vfs->xorg_log, 'MALI(') !== false)
+			{
+				$info = 'ARM Mali';
+			}
+		}
+
 		if(($bracket_open = strpos($info, '[')) !== false)
 		{
 			// Report only the information inside the brackets if it's more relevant...
@@ -1374,7 +1392,7 @@ class phodevi_gpu extends phodevi_device_interface
 			}
 		}
 
-		if(stripos($info, 'NVIDIA') === false && (stripos($info, 'Quadro') !== false || stripos($info, 'GeForce') !== false))
+		if(stripos($info, 'NVIDIA') === false && (stripos($info, 'Quadro') !== false || stripos($info, 'Titan ') !== false ||  stripos($info, ' Tesla') !== false || stripos($info, 'GeForce') !== false || substr($info, 0, 2) == 'NV'))
 		{
 			$info = 'NVIDIA' . ' ' . $info;
 		}
@@ -1384,19 +1402,47 @@ class phodevi_gpu extends phodevi_device_interface
 			$info = 'AMD ' . $info;
 		}
 
-		if(phodevi::is_linux() && ($vendor = phodevi_linux_parser::read_pci_subsystem_value('VGA compatible controller')) != null && stripos($info, $vendor) === false && (stripos($info, 'AMD') !== false || stripos($info, 'NVIDIA') !== false))
+		if(phodevi::is_linux() && ($vendor = phodevi_linux_parser::read_pci_subsystem_value('VGA compatible controller')) != null && stripos($info, $vendor) === false && (stripos($info, 'AMD') !== false || stripos($info, 'NVIDIA') !== false || stripos($info, 'Intel') !== false))
 		{
 			$info = $vendor . ' ' . $info;
 		}
 
-		if($video_ram > 64 && strpos($info, $video_ram) == false) // assume more than 64MB of vRAM
-		{
-			$info .= ' ' . $video_ram . 'MB';
-		}
-	
 		$clean_phrases = array('OpenGL Engine');
 		$info = str_replace($clean_phrases, null, $info);
 
+		if(!empty($info) && $video_ram > 64 && strpos($info, $video_ram) == false && stripos($info, 'llvmpipe') === false) // assume more than 64MB of vRAM
+		{
+			if($video_ram < 1024)
+			{
+				$info .= ' ' . $video_ram . 'MB';
+			}
+			else
+			{
+				$video_ram = round($video_ram / 1024) . 'GB';
+				if(strpos($info, $video_ram) == false)
+				{
+					$info .= ' ' . $video_ram;
+				}
+			}
+		}
+
+		if(empty($info) && is_dir('/sys/class/drm/card0/device/driver/pvrsrvkm'))
+		{
+			$info = 'PowerVR';
+		}
+		if(empty($info) && is_readable('/sys/class/graphics/fb0/name'))
+		{
+			// Last possible fallback...
+			$info = str_replace(' FB', '', pts_file_io::file_get_contents('/sys/class/graphics/fb0/name'));
+		}
+
+		// Happens with Intel Iris Gallium3D
+		$info = str_replace('Mesa ', ' ', $info);
+		/*if(empty($info))
+		{
+			$info = 'Unknown';
+		}*/
+
 		return $info;
 	}
 }
