@@ -3,8 +3,8 @@
 /*
 	Phoronix Test Suite
 	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
-	Copyright (C) 2008 - 2013, Phoronix Media
-	Copyright (C) 2008 - 2013, Michael Larabel
+	Copyright (C) 2008 - 2020, Phoronix Media
+	Copyright (C) 2008 - 2020, Michael Larabel
 	phodevi_linux_parser.php: General parsing functions specific to Linux
 
 	This program is free software; you can redistribute it and/or modify
@@ -23,29 +23,56 @@
 
 class phodevi_linux_parser
 {
-	public static function read_ipmitool_sensor($sensor)
+	public static function read_ipmitool_sensor($sensors, $default_value = false)
 	{
-		$value = false;
+		$value = $default_value;
 		$ipmitool = shell_exec('ipmitool sdr list 2>&1');
 
+		foreach(pts_arrays::to_array($sensors) as $sensor)
+		{
+			$hit = stripos($ipmitool, $sensor);
+
+			if($hit !== false)
+			{
+				$trimmed = substr($ipmitool, ($hit + strlen($sensor)));
+				$trimmed = substr($trimmed, 0, strpos($trimmed, PHP_EOL));
+				$trimmed = explode('|', $trimmed);
+
+				if(count($trimmed) == 3)
+				{
+					$value = explode(' ', trim($trimmed[1]));
+					$value = $value[0];
+					break;
+				}
+			}
+		}
+
+		return $value;
+	}
+	public static function read_ipmitool_dcmi_power()
+	{
+		$value = false;
+		$ipmitool = shell_exec('ipmitool dcmi power reading 2>&1');
+
+		$sensor = "Instantaneous power reading:";
 		$hit = stripos($ipmitool, $sensor);
 
 		if($hit !== false)
 		{
 			$trimmed = substr($ipmitool, ($hit + strlen($sensor)));
 			$trimmed = substr($trimmed, 0, strpos($trimmed, PHP_EOL));
-			$trimmed = explode('|', $trimmed);
+			$trimmed = trim($trimmed);
+			$trimmed = explode(' ', $trimmed);
 
-			if(count($trimmed) == 3)
+			if(count($trimmed) == 2)
 			{
-				$value = explode(' ', trim($trimmed[1]));
-				$value = $value[0];
+				$value = $trimmed[0];
 			}
 		}
 
 		return $value;
 	}
-	public static function read_sysfs_node($search, $type = 'NUMERIC', $node_dir_check = null, $find_position = 1)
+	public static function read_sysfs_node($search, $type = 'NUMERIC', $node_dir_check = null, $find_position = 1, $return_node = false)
 	{
 		static $sysfs_file_cache = null;
 		$arg_hash = crc32(serialize(func_get_args()));
@@ -113,7 +140,7 @@ class phodevi_linux_parser
 						}
 						break;
 					case 'POSITIVE_NUMERIC':
-						if(is_numeric($sysfs_value) && $sysfs_value > 0)
+						if(is_numeric($sysfs_value) && $sysfs_value >= 0)
 						{
 							$sysfs_file_cache[$arg_hash] = $sysfs_file;
 						}
@@ -147,6 +174,11 @@ class phodevi_linux_parser
 			}
 		}
 
+		if($return_node)
+		{
+			return $sysfs_file_cache[$arg_hash] == false ? -1 : $sysfs_file;
+		}
+
 		return $sysfs_file_cache[$arg_hash] == false ? -1 : pts_file_io::file_get_contents($sysfs_file_cache[$arg_hash]);
 	}
 	public static function read_dmidecode($type, $sub_type, $object, $find_once = false, $ignore = null)
@@ -154,9 +186,9 @@ class phodevi_linux_parser
 		// Read Linux dmidecode
 		$value = array();
 
-		if(is_readable('/dev/mem') && pts_client::executable_in_path('dmidecode'))
+		if((phodevi::is_root() || is_readable('/dev/mem')) && pts_client::executable_in_path('dmidecode'))
 		{
-			$ignore = pts_arrays::to_array($ignore);
+			$ignore = array_map('strtolower', pts_arrays::to_array($ignore));
 
 			$dmidecode = shell_exec('dmidecode --type ' . $type . ' 2>&1');
 			$sub_type = "\n" . $sub_type . "\n";
@@ -177,7 +209,7 @@ class phodevi_linux_parser
 					{
 						$dmidecode_r = pts_strings::colon_explode($dmidecode_elements[$i]);
 
-						if($dmidecode_r[0] == $object && isset($dmidecode_r[1]) && !in_array($dmidecode_r[1], $ignore))
+						if($dmidecode_r[0] == $object && isset($dmidecode_r[1]) && !in_array(strtolower($dmidecode_r[1]), $ignore))
 						{
 							array_push($value, $dmidecode_r[1]);
 							$found_in_section = true;
@@ -201,7 +233,8 @@ class phodevi_linux_parser
 	}
 	public static function read_sys_disk_speed($path, $to_read)
 	{
-		$speed = -1; // in MB/s
+		$delta_mb = -1; // in MB/s
+		$measure_time = 1000000; // microseconds
 
 		if(is_file($path))
 		{
@@ -209,32 +242,27 @@ class phodevi_linux_parser
 			{
 				case 'WRITE':
 					$sector = 6;
-					$time = 7;
 					break;
 				case 'READ':
 					$sector = 2;
-					$time = 3;
 					break;
 				default:
-					return $speed;
+					return $delta_mb;
 					break;
 			}
 
 			$start_stat = pts_strings::trim_spaces(file_get_contents($path));
-			usleep(500000);
+			usleep($measure_time);
 			$end_stat = pts_strings::trim_spaces(file_get_contents($path));
 
 			$start_stat = explode(' ', $start_stat);
 			$end_stat = explode(' ', $end_stat);
 
 			$delta_sectors = $end_stat[$sector] - $start_stat[$sector];
-			$delta_ms_spent = $end_stat[$time] - $start_stat[$time];
 
-			// assuming 512 byte sectors
+			// TODO check sector size instead of hardcoding it
 			$delta_mb = $delta_sectors * 512 / 1048576;
-			$delta_seconds = $delta_ms_spent / 1000;
-
-			$speed = $delta_seconds != 0 ? $delta_mb / $delta_seconds : 0;
+			$speed = $delta_mb * 1000000 / $measure_time;
 		}
 
 		return pts_math::set_precision($speed, 2);
@@ -264,239 +292,6 @@ class phodevi_linux_parser
 
 		return $dmi;
 	}
-	public static function read_ati_overdrive($attribute, $adapter = 0)
-	{
-		// Read ATI OverDrive information
-		// OverDrive supported in fglrx 8.52+ drivers
-		$value = false;
-
-		if(($amdconfig = self::find_amdconfig()))
-		{
-			if($attribute == 'Temperature')
-			{
-				$info = shell_exec($amdconfig . ' --adapter=' . $adapter . ' --od-gettemperature 2>&1');
-
-				if(($start = strpos($info, 'Temperature -')) !== false)
-				{
-					$info = substr($info, $start + 14);
-					$value = substr($info, 0, strpos($info, ' C'));
-				}
-			}
-			else if($attribute == 'FanSpeed')
-			{
-				// Right now there is no standardized interface to get the fan speed through besides the pplib command
-				$info = shell_exec($amdconfig . ' --adapter=' . $adapter . ' --pplib-cmd \'get fanspeed 0\' 2>&1');
-
-				if(($start = strpos($info, 'Fan Speed:')) !== false)
-				{
-					$info = substr($info, $start + 11);
-					$info = substr($info, 0, strpos($info, '%'));
-
-					if(is_numeric($info))
-					{
-						$value = $info;
-					}
-				}
-			}
-			else
-			{
-				$info = shell_exec($amdconfig . ' --adapter=' . $adapter . ' --od-getclocks 2>&1');
-
-				if(strpos($info, 'GPU') !== false)
-				{
-					foreach(explode("\n", $info) as $line)
-					{
-						$line_r = pts_strings::colon_explode($line);
-
-						if(count($line_r) == 2)
-						{
-							$od_option = str_replace(' ', null, $line_r[0]);
-
-							if($od_option == $attribute)
-							{
-								$od_value = pts_strings::trim_spaces($line_r[1]);
-								$od_value = str_replace(array('%'), null, $od_value);
-								$od_value_r = explode(' ', $od_value);
-
-								$value = (count($od_value_r) == 1 ? $od_value_r[0] : $od_value_r);			
-							}
-						}
-					}
-				}
-			}
-		}
-
-		return $value;
-	}
-	public static function read_amd_pcsdb($attribute)
-	{
-		// Read AMD's AMDPCSDB, AMD Persistent Configuration Store Database
-		static $try_aticonfig = true;
-		static $is_first_read = true;
-		$ati_info = null;
-
-		if($try_aticonfig)
-		{
-			if(($amdconfig = self::find_amdconfig()))
-			{
-				$info = shell_exec($amdconfig . ' --get-pcs-key=' . $attribute . ' 2>&1');
-
-				if($is_first_read && strpos($info, 'No supported adapters') != false)
-				{
-					$try_aticonfig = false;
-				}
-				else
-				{
-					if(($pos = strpos($info, ':')) > 0 && strpos($info, 'Error') === false)
-					{
-						$ati_info = substr($info, $pos + 2);
-						$ati_info = substr($ati_info, 0, strpos($ati_info, ' '));
-					}
-				}
-			}
-			else
-			{
-				$try_aticonfig = false;
-			}
-
-			$is_first_read = false;
-		}
-
-		if($ati_info == null && is_file('/etc/ati/amdpcsdb'))
-		{
-			// Using aticonfig --get-pcs-key failed, switch to the PTS direct parser of AMDPCSDB
-			$ati_info = phodevi_linux_parser::read_amd_pcsdb_direct_parser($attribute);
-		}
-
-		return $ati_info;
-	}
-	public static function read_amd_pcsdb_direct_parser($attribute, $find_once = false)
-	{
-		// Read AMD's AMDPCSDB, AMD Persistent Configuration Store Database but using our own internal parser instead of relying upon aticonfig/amdconfig
-		$amdpcsdb_file = null;
-		$last_found_section_count = -1;
-		$this_section_count = 0;
-		$attribute_values = array();
-		$attribute = pts_strings::comma_explode($attribute);
-
-		if(count($attribute) == 2)
-		{
-			$attribute_prefix = array_reverse(explode('/', $attribute[0]));
-			$attribute_key = $attribute[1];
-			$is_in_prefix = false;
-
-			if(is_file('/etc/ati/amdpcsdb'))
-			{
-				$amdpcsdb_file = explode("\n", file_get_contents('/etc/ati/amdpcsdb'));
-			}
-
-			for($l = 0; $l < count($amdpcsdb_file) && ($find_once == false || $last_found_section_count == -1); $l++)
-			{
-				$line = trim($amdpcsdb_file[$l]);
-
-				if(substr($line, 0, 1) == '[' && substr($line, -1) == ']')
-				{
-					// AMDPCSDB Header
-					$prefix_matches = true;
-					$header = array_reverse(explode('/', substr($line, 1, -1)));
-
-					for($i = 0; $i < count($attribute_prefix) && $i < count($header) && $prefix_matches; $i++)
-					{
-						if($attribute_prefix[$i] != $header[$i] && pts_strings::proximity_match($attribute_prefix[$i], $header[$i]) == false)
-						{
-							$prefix_matches = false;
-						}
-					}
-
-					if($prefix_matches)
-					{
-						$is_in_prefix = true;
-						$this_section_count++;
-					}
-					else
-					{
-						$is_in_prefix = false;
-					}
-				}
-				else if($is_in_prefix && $this_section_count != $last_found_section_count && count(($key_components = explode('=', $line))) == 2)
-				{
-					// AMDPCSDB Value
-					if($key_components[0] == $attribute_key)
-					{
-						$value_type = substr($key_components[1], 0, 1);
-						$value = substr($key_components[1], 1);
-
-						switch($value_type)
-						{
-							case 'V':
-								// Value
-								if(is_numeric($value) && strlen($value) < 9)
-								{
-									$value = dechex($value);
-									$value = '0x' . str_repeat(0, 8 - strlen($value)) . strtoupper($value);
-								}
-							break;
-							case 'R':
-								// Raw
-							break;
-							case 'S':
-								// String
-							break;
-
-						}
-						array_push($attribute_values, $value);
-						$last_found_section_count = $this_section_count;
-					}
-				}
-			}
-		}
-
-		if(count($attribute_values) == 0)
-		{
-			$attribute_values = null;
-		}
-		else if(count($attribute_values) == 1)
-		{
-			$attribute_values = $attribute_values[0];
-		}
-
-		return $attribute_values;
-	}
-	public static function find_amdconfig()
-	{
-		$amdconfig = false;
-
-		if(($t = pts_client::executable_in_path('aticonfig')))
-		{
-			$amdconfig = $t;
-		}
-		else if(($t = pts_client::executable_in_path('amdconfig')))
-		{
-			$amdconfig = $t;
-		}
-
-		return $amdconfig;
-	}
-	public static function read_amd_graphics_adapters()
-	{
-		// Read ATI/AMD graphics hardware using aticonfig
-		$adapters = array();
-
-		if(($amdconfig = self::find_amdconfig()))
-		{
-			$info = trim(shell_exec($amdconfig . ' --list-adapters 2>&1'));
-
-			foreach(explode("\n", $info) as $line)
-			{
-				if(($last_point = strrpos($line, '.')) > 0)
-				{
-					array_push($adapters, substr($line, $last_point + 3));
-				}
-			}
-		}
-
-		return $adapters;
-	}
 	public static function read_cpuinfo($attribute, $cpuinfo = false)
 	{
 		// Read CPU information
@@ -520,7 +315,7 @@ class phodevi_linux_parser
 
 			foreach($cpuinfo_lines as $line)
 			{
-				$line = pts_strings::trim_explode(': ', $line);
+				$line = pts_strings::trim_explode(': ', $line, 2);
 
 				if(!isset($line[0]))
 				{
@@ -544,6 +339,57 @@ class phodevi_linux_parser
 
 		return $cpuinfo_matches;
 	}
+	public static function systemctl_active($service)
+	{
+		$active = false;
+		if(pts_client::executable_in_path('systemctl'))
+		{
+			$is_active = trim(shell_exec('systemctl is-active ' . $service . ' 2>/dev/null'));
+			if($is_active == 'active')
+			{
+				$active = true;
+			}
+		}
+		return $active;
+	}
+	public static function cpuinfo_to_array($cpuinfo = null)
+	{
+		if($cpuinfo == null && is_file('/proc/cpuinfo'))
+		{
+			$cpuinfo = file_get_contents('/proc/cpuinfo');
+		}
+		
+		$cpuinfo_lines = explode("\n", $cpuinfo);
+		$cpuinfo_r = array();
+
+		foreach(explode("\n", $cpuinfo) as $line)
+		{
+			$line = pts_strings::trim_explode(': ', $line, 2);
+			if(!isset($line[0]))
+			{
+				continue;
+			}
+
+			$this_attribute = $line[0];
+			$this_value = trim(count($line) > 1 ? $line[1] : null);
+			if(in_array($this_attribute, array('flags', 'bugs', 'power management')))
+			{
+				$this_value = explode(' ', $this_value);
+			}
+			$cpuinfo_r[$this_attribute] = $this_value;
+		}
+		
+		return $cpuinfo_r;
+	}
+	public static function read_cpuinfo_single($attribute, $cpuinfo = false)
+	{
+		$cpuinfo = self::read_cpuinfo($attribute, $cpuinfo);
+		if(!empty($cpuinfo))
+		{
+			return array_pop($cpuinfo);
+		}
+		return null;
+	}
 	public static function read_lsb_distributor_id()
 	{
 		$vendor = phodevi_linux_parser::read_lsb('Distributor ID');
@@ -696,7 +542,7 @@ class phodevi_linux_parser
 
 			if(($pos = strpos($pci_info, $desc[$i])) !== false)
 			{
-				$sub_pci_info = str_replace(array('[AMD]', '[AMD/ATI]'), null, substr($pci_info, $pos + strlen($desc[$i])));
+				$sub_pci_info = str_replace(array('[AMD]', '[AMD/ATI]', ' Limited'), null, substr($pci_info, $pos + strlen($desc[$i])));
 				$EOL = strpos($sub_pci_info, "\n");
 
 				if($clean_string)
@@ -731,6 +577,76 @@ class phodevi_linux_parser
 
 		return $info;
 	}
+	public static function read_pci_multi($desc, $clean_string = true)
+	{
+		// Read PCI bus information
+		static $pci_info = null;
+		$info = array();
+		$desc = pts_arrays::to_array($desc);
+
+		if($pci_info == null)
+		{
+			if(!is_executable('/usr/bin/lspci') && is_executable('/sbin/lspci'))
+			{
+				$lspci_cmd = '/sbin/lspci';
+			}
+			else if(($lspci = pts_client::executable_in_path('lspci')))
+			{
+				$lspci_cmd = $lspci;
+			}
+			else
+			{
+				return false;
+			}
+
+			$pci_info = shell_exec($lspci_cmd . ' 2> /dev/null');
+		}
+
+		for($i = 0; $i < count($desc); $i++)
+		{
+			if(substr($desc[$i], -1) != ':')
+			{
+				$desc[$i] .= ':';
+			}
+			$pos = 0;
+			while(($pos = strpos($pci_info, $desc[$i], $pos)) !== false)
+			{
+				$pos += strlen($desc[$i]);
+				$sub_pci_info = str_replace(array('[AMD]', '[AMD/ATI]', ' Limited', ' Connection', ' Gigabit', ' Wireless', '(1)', '(2)', '(3)', '(4)', '(5)', '(6)', '(7)', '(8)', '(9)'), null, substr($pci_info, $pos));
+				$EOL = strpos($sub_pci_info, "\n");
+
+				if($clean_string)
+				{
+					if(($temp = strpos($sub_pci_info, '/')) < $EOL && $temp > 0)
+					{
+						if(($temp = strpos($sub_pci_info, ' ', ($temp + 2))) < $EOL && $temp > 0)
+						{
+							$EOL = $temp;
+						}
+					}
+
+					if(($temp = strpos($sub_pci_info, '(')) < $EOL && $temp > 0)
+					{
+						$EOL = $temp;
+					}
+
+					if(($temp = strpos($sub_pci_info, '[')) < $EOL && $temp > 0)
+					{
+						$EOL = $temp;
+					}
+				}
+
+				$sub_pci_info = trim(substr($sub_pci_info, 0, $EOL));
+
+				if(($strlen = strlen($sub_pci_info)) >= 6 && $strlen < 128)
+				{
+					$info[] = pts_strings::strip_string($sub_pci_info);
+				}
+			}
+		}
+
+		return $info;
+	}
 	public static function read_sensors($attributes)
 	{
 		// Read LM_Sensors
