@@ -3,8 +3,8 @@
 /*
 	Phoronix Test Suite
 	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
-	Copyright (C) 2010 - 2014, Phoronix Media
-	Copyright (C) 2010 - 2014, Michael Larabel
+	Copyright (C) 2010 - 2019, Phoronix Media
+	Copyright (C) 2010 - 2019, Michael Larabel
 	pts_OverviewGraph.php: A graping object to create an "overview" / mini graphs of a pts_result_file for pts_Graph
 
 	This program is free software; you can redistribute it and/or modify
@@ -21,9 +21,9 @@
 	along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
 
-class pts_OverviewGraph extends pts_Graph
+class pts_OverviewGraph extends pts_graph_core
 {
-	protected $result_file;
+	protected $result_file = null;
 
 	protected $system_identifiers;
 	protected $test_titles;
@@ -33,8 +33,6 @@ class pts_OverviewGraph extends pts_Graph
 	protected $graph_row_height = 120;
 	protected $graph_row_count;
 
-	public $skip_graph = false;
-
 	public function __construct($result_file)
 	{
 		$result_object = null;
@@ -44,7 +42,6 @@ class pts_OverviewGraph extends pts_Graph
 		if($result_file->is_multi_way_comparison())
 		{
 			// Multi way comparisons currently render the overview graph as blank
-			$this->skip_graph = true;
 			return;
 		}
 
@@ -52,16 +49,19 @@ class pts_OverviewGraph extends pts_Graph
 		if(count($this->system_identifiers) < 2)
 		{
 			// No point in generating this when there is only one identifier
-			$this->skip_graph = true;
 			return;
 		}
 
 		$result_objects = array();
+		$test_titles = array();
 		foreach($result_file->get_result_objects() as $result_object)
 		{
 			if($result_object->test_profile->get_display_format() == 'BAR_GRAPH')
 			{
-				array_push($result_objects, $result_object);
+				$result_objects[] = $result_object;
+				$test_titles[] = $result_object->test_profile->get_title();
+				foreach($result_object->test_result_buffer->buffer_items as &$buffer_item)
+				pts_arrays::unique_push($this->graph_identifiers, $buffer_item->get_result_identifier());
 			}
 		}
 
@@ -69,7 +69,6 @@ class pts_OverviewGraph extends pts_Graph
 		if($result_object_count < 3)
 		{
 			// No point in generating this if there aren't many tests
-			$this->skip_graph = true;
 			return;
 		}
 		$result_file->override_result_objects($result_objects);
@@ -78,8 +77,7 @@ class pts_OverviewGraph extends pts_Graph
 		$this->i['identifier_size'] = 6.5;
 		$this->i['graph_width'] = 1000;
 
-		$titles = $result_file->get_test_titles();
-		list($longest_title_width, $longest_title_height) = pts_svg_dom::estimate_text_dimensions(pts_strings::find_longest_string($titles), $this->i['identifier_size']);
+		list($longest_title_width, $longest_title_height) = pts_svg_dom::estimate_text_dimensions(pts_strings::find_longest_string($test_titles), $this->i['identifier_size']);
 
 		$this->i['left_start'] += 20;
 		$this->graphs_per_row = min((count($this->system_identifiers) > 10 ? 6 : 10), floor(($this->i['graph_width'] - $this->i['left_start'] - $this->i['left_end_right']) / ($longest_title_width + 4)));
@@ -89,8 +87,8 @@ class pts_OverviewGraph extends pts_Graph
 		$this->i['top_start'] += 20 + (count($this->system_identifiers) / 3 * $this->i['identifier_size']);
 		$height = $this->i['top_start'] + ($this->graph_row_count * ($this->graph_row_height + 15));
 
-		$this->graph_title = $result_file->get_title();
-		$this->graph_y_title = null;
+		$this->i['graph_title'] = $result_file->get_title();
+		$this->i['graph_y_title'] = null;
 		$this->i['graph_proportion'] = 'HIB';
 		$this->i['show_background_lines'] = true;
 
@@ -99,18 +97,17 @@ class pts_OverviewGraph extends pts_Graph
 
 		return true;
 	}
-	public function doSkipGraph()
-	{
-		return $this->skip_graph;
-	}
 	public function renderGraph()
 	{
-		$this->graph_data_title = &$this->system_identifiers;
+		if($this->result_file == null)
+		{
+			return false;
+		}
 		$this->i['graph_max_value'] = 1.0;
 		$l_height = 15;
 		$this->i['key_line_height'] = $l_height;
 
-		if(($key_count = count($this->graph_data_title)) > 8)
+		if(($key_count = count($this->system_identifiers)) > 8)
 		{
 			$this->update_graph_dimensions(-1, $this->i['graph_height'] + (floor(($key_count - 8) / 4) * 14), true);
 		}
@@ -163,7 +160,7 @@ class pts_OverviewGraph extends pts_Graph
 					switch($result_object->test_profile->get_result_proportion())
 					{
 						case 'HIB':
-							$value = $buffer_item->get_result_value() / $divide_value;
+							$value = $divide_value != 0 ? $buffer_item->get_result_value() / $divide_value : 0;
 							break;
 						case 'LIB':
 							$value = $divide_value / $buffer_item->get_result_value();
@@ -195,13 +192,15 @@ class pts_OverviewGraph extends pts_Graph
 		if($has_graphed_a_bar == false)
 		{
 			// Don't show an empty overview graph...
-			$this->skip_graph = true;
+			//$this->skip_graph = true;
 		}
 
 
 		//$this->render_graph_base($this->i['left_start'], $this->i['top_start'], $this->i['graph_left_end'], $this->i['graph_top_end']);
 		$this->render_graph_heading();
 		//$this->render_graph_watermark();
+
+		return true;
 	}
 }
 
