@@ -3,8 +3,8 @@
 /*
 	Phoronix Test Suite
 	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
-	Copyright (C) 2011 - 2014, Phoronix Media
-	Copyright (C) 2011 - 2014, Michael Larabel
+	Copyright (C) 2011 - 2017, Phoronix Media
+	Copyright (C) 2011 - 2017, Michael Larabel
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -32,18 +32,15 @@ class pts_svg_dom
 		$dom = new DOMImplementation();
 		$dtd = $dom->createDocumentType('svg', '-//W3C//DTD SVG 1.1//EN', 'http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd');
 		$this->dom = $dom->createDocument(null, null, $dtd);
-		$this->dom->formatOutput = PTS_IS_CLIENT;
+		$this->dom->formatOutput = PTS_IS_CLIENT && PTS_IS_DEV_BUILD;
 
-		if(stripos(PTS_PHP_VERSION, 'hiphop') === false)
-		{
-			$pts_comment = $this->dom->createComment(pts_title(true) . ' [ http://www.phoronix-test-suite.com/ ]');
-			$this->dom->appendChild($pts_comment);
-		}
+		$pts_comment = $this->dom->createComment(pts_core::program_title(false) . ' [ http://www.phoronix-test-suite.com/ ]');
+		$this->dom->appendChild($pts_comment);
 
 		$this->svg = $this->dom->createElementNS('http://www.w3.org/2000/svg', 'svg');
 		$this->svg->setAttribute('xmlns:xlink', 'http://www.w3.org/1999/xlink');
 		$this->svg->setAttribute('version', '1.1');
-		$this->svg->setAttribute('font-family', 'sans-serif');
+		$this->svg->setAttribute('font-family', 'sans-serif, droid-sans, helvetica, verdana, tahoma');
 		$this->svg->setAttribute('viewbox', '0 0 ' . $width . ' ' . $height);
 		$this->svg->setAttribute('width', $width);
 		$this->svg->setAttribute('height', $height);
@@ -58,9 +55,8 @@ class pts_svg_dom
 		// XXX: Alias for output. With PTS 3.8 this is just here for API compatibility with OpenBenchmarking.org.
 		$this->output($save_as, $format);
 	}
-	public function output($save_as = null, &$format = null)
+	public function output($save_as = null, $output_format = 'SVG')
 	{
-		$output_format = 'SVG';
 		if(isset($_SERVER['HTTP_USER_AGENT']) || isset($_REQUEST['force_format']))
 		{
 			static $browser_renderer = null;
@@ -83,6 +79,7 @@ class pts_svg_dom
 
 		switch($output_format)
 		{
+			case 'JPG':
 			case 'JPEG':
 				$output = pts_svg_dom_gd::svg_dom_to_gd($this->dom, 'JPEG');
 				$output_format = 'jpg';
@@ -91,6 +88,11 @@ class pts_svg_dom
 				$output = pts_svg_dom_gd::svg_dom_to_gd($this->dom, 'PNG');
 				$output_format = 'png';
 				break;
+			case 'HTML':
+				$html = new pts_svg_dom_html($this->dom);
+				$output = $html->get_html();
+				$output_format = 'html';
+				break;
 			case 'SVG':
 			default:
 				$output = $this->save_xml();
@@ -124,6 +126,7 @@ class pts_svg_dom
 	}
 	public function draw_svg_line($start_x, $start_y, $end_x, $end_y, $color, $line_width = 1, $extra_elements = null)
 	{
+		// this function is equivalent to $this->svg_dom->add_element('line', array('x1' => , 'y1' => , 'x2' => , 'y2' => , 'stroke' => , 'stroke-width' => ));
 		$attributes = array('x1' => $start_x, 'y1' => $start_y, 'x2' => $end_x, 'y2' => $end_y, 'stroke' => $color, 'stroke-width' => $line_width);
 
 		if($extra_elements != null)
@@ -155,7 +158,23 @@ class pts_svg_dom
 		$extra_attributes['fill'] = $color;
 		$this->add_element('circle', $extra_attributes);
 	}
-	public function add_element($element_type, $attributes = array())
+	public function make_a($url)
+	{
+		$el = $this->dom->createElement('a');
+		$el->setAttribute('xlink:href', $url);
+		$el->setAttribute('target', '_blank');
+		return $this->svg->appendChild($el);
+	}
+	public function make_g($attributes = array(), $append_to = false)
+	{
+		$el = $this->dom->createElement('g');
+		foreach($attributes as $name => $value)
+		{
+			$el->setAttribute($name, $value);
+		}
+		return $append_to ? $append_to->appendChild($el) : $this->svg->appendChild($el);
+	}
+	public function add_element($element_type, $attributes = array(), $append_to = false)
 	{
 		$el = $this->dom->createElement($element_type);
 
@@ -165,14 +184,28 @@ class pts_svg_dom
 			$link_key = ($element_type == 'image' ? 'http_link' : 'xlink:href');
 			$link = $this->dom->createElement('a');
 			$link->setAttribute('xlink:href', $attributes[$link_key]);
-			$link->setAttribute('xlink:show', 'new');
+			$link->setAttribute('target', '_blank');
 			$link->appendChild($el);
-			$this->svg->appendChild($link);
+			if($append_to)
+			{
+				$append_to->appendChild($link);
+			}
+			else
+			{
+				$this->svg->appendChild($link);
+			}
 			unset($attributes[$link_key]);
 		}
 		else
 		{
-			$this->svg->appendChild($el);
+			if($append_to)
+			{
+				$append_to->appendChild($el);
+			}
+			else
+			{
+				$this->svg->appendChild($el);
+			}
 		}
 
 		foreach($attributes as $name => $value)
@@ -180,7 +213,7 @@ class pts_svg_dom
 			$el->setAttribute($name, $value);
 		}
 	}
-	public function add_text_element($text_string, $attributes)
+	public function add_text_element($text_string, $attributes, $append_to = false)
 	{
 		$el = $this->dom->createElement('text');
 		$text_node = $this->dom->createTextNode($text_string);
@@ -190,19 +223,33 @@ class pts_svg_dom
 		{
 			$link = $this->dom->createElement('a');
 			$link->setAttribute('xlink:href', $attributes['xlink:href']);
-			$link->setAttribute('xlink:show', 'new');
+			$link->setAttribute('target', '_blank');
 			$link->appendChild($el);
-			$this->svg->appendChild($link);
+			if($append_to)
+			{
+				$append_to->appendChild($link);
+			}
+			else
+			{
+				$this->svg->appendChild($link);
+			}
 			unset($attributes['xlink:href']);
 		}
 		else
 		{
-			$this->svg->appendChild($el);
+			if($append_to)
+			{
+				$append_to->appendChild($el);
+			}
+			else
+			{
+				$this->svg->appendChild($el);
+			}
 		}
 
 		foreach($attributes as $name => $value)
 		{
-			if($value === null)
+			if($value == null && $value !== 0)
 			{
 				continue;
 			}
@@ -210,7 +257,7 @@ class pts_svg_dom
 			$el->setAttribute($name, $value);
 		}
 	}
-	public function add_textarea_element($text_string, $attributes, &$estimated_height = 0)
+	public function add_textarea_element($text_string, $attributes, &$estimated_height = 0, &$append_to = false)
 	{
 		if(!isset($attributes['width']))
 		{
@@ -221,6 +268,7 @@ class pts_svg_dom
 		if($queue_dimensions[0] < $attributes['width'])
 		{
 			// No wrapping is occuring, so stuff it in a more efficient text element instead
+			unset($attributes['width']);
 			$this->add_text_element($text_string, $attributes);
 			$estimated_height += ($attributes['font-size'] + 3);
 			return;
@@ -266,14 +314,28 @@ class pts_svg_dom
 		{
 			$link = $this->dom->createElement('a');
 			$link->setAttribute('xlink:href', $attributes['xlink:href']);
-			$link->setAttribute('xlink:show', 'new');
+			$link->setAttribute('target', '_blank');
 			$link->appendChild($el);
-			$this->svg->appendChild($link);
+			if($append_to)
+			{
+				$append_to->appendChild($link);
+			}
+			else
+			{
+				$this->svg->appendChild($link);
+			}
 			unset($attributes['xlink:href']);
 		}
 		else
 		{
-			$this->svg->appendChild($el);
+			if($append_to)
+			{
+				$append_to->appendChild($el);
+			}
+			else
+			{
+				$this->svg->appendChild($el);
+			}
 		}
 
 		foreach($attributes as $name => $value)
