@@ -3,8 +3,8 @@
 /*
 	Phoronix Test Suite
 	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
-	Copyright (C) 2008 - 2011, Phoronix Media
-	Copyright (C) 2008 - 2011, Michael Larabel
+	Copyright (C) 2008 - 2019, Phoronix Media
+	Copyright (C) 2008 - 2019, Michael Larabel
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -29,22 +29,30 @@ class pts_test_file_download
 	private $filename;
 	private $filesize;
 	private $md5;
+	private $sha256;
 	private $architecture;
 	private $platform;
+	private $is_optional = false;
 
 	private $download_location_type = null;
 	private $download_location_path = null;
 
-	public function __construct($url = null, $filename = null, $filesize = 0, $md5 = null, $platform = null, $architecture = null)
+	public function __construct($url = null, $filename = null, $filesize = 0, $md5 = null, $sha256 = null, $platform = null, $architecture = null, $is_optional = false)
 	{
 		$this->filename = empty($filename) ? basename($url) : $filename;
 		$this->url = $this->filename == $url ? null : $url;
 		$this->filesize = !is_numeric($filesize) ? 0 : $filesize;
 		$this->md5 = $md5;
-		$this->location_type = null;
-		$this->location_path = array();
+		$this->sha256 = $sha256;
 		$this->platform = $platform;
 		$this->architecture = $architecture;
+		$this->is_optional = $is_optional || strtolower($is_optional) == 'true';
+
+		if(phodevi::is_windows() || !extension_loaded('openssl'))
+		{
+			// Windows with PHP stock binaries has problems downloading from HTTPS
+			$this->url = str_replace('https://', 'http://', $this->url);
+		}
 
 		// Check for longest file name length as the text UI takes advantage of it
 
@@ -90,6 +98,45 @@ class pts_test_file_download
 	{
 		return $this->md5;
 	}
+	public function get_sha256()
+	{
+		return $this->sha256;
+	}
+	public function is_optional()
+	{
+		return $this->is_optional == true;
+	}
+	public function is_optional_string()
+	{
+		return $this->is_optional ? 'TRUE' : '';
+	}
+	public function check_file_hash($file)
+	{
+		if(!is_file($file))
+		{
+			return false;
+		}
+		else if(pts_client::read_env('NO_FILE_HASH_CHECKS') != false || pts_client::read_env('NO_MD5_CHECKS') != false)
+		{
+			return true;
+		}
+		else if($this->sha256 && function_exists('hash_file'))
+		{
+			return hash_file('sha256', $file) == $this->sha256;
+		}
+		else if($this->md5)
+		{
+			return md5_file($file) == $this->md5;
+		}
+		else if(filesize($file) > 0)
+		{
+			return true;
+		}
+		else
+		{
+			return false;
+		}
+	}
 	public function set_filesize($size)
 	{
 		$this->filesize = is_numeric($size) ? $size : 0;
@@ -99,6 +146,7 @@ class pts_test_file_download
 		// IN_DESTINATION_DIR == already good, in the destination directory already, was previously downloaded
 		// LOCAL_DOWNLOAD_CACHE == In a local download cache, can be copied, etc
 		// REMOTE_DOWNLOAD_CACHE == In a remote download cache for download
+		// LOOKASIDE_DOWNLOAD_CACHE == In another test installation directory
 
 		$this->download_location_type = $location_type;
 		$this->download_location_path = $location_path;
