@@ -3,8 +3,8 @@
 /*
 	Phoronix Test Suite
 	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
-	Copyright (C) 2008 - 2014, Phoronix Media
-	Copyright (C) 2008 - 2014, Michael Larabel
+	Copyright (C) 2008 - 2021, Phoronix Media
+	Copyright (C) 2008 - 2021, Michael Larabel
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -23,46 +23,87 @@
 class pts_test_profile extends pts_test_profile_parser
 {
 	public $test_installation = false;
+	protected static $test_installation_cache;
 
-	public function __construct($identifier = null, $override_values = null)
+	public function __construct($identifier = null, $override_values = null, $normal_init = true)
 	{
-		parent::__construct($identifier);
+		parent::__construct($identifier, $normal_init);
 
 		if($override_values != null && is_array($override_values))
 		{
-			$this->xml_parser->overrideXMLValues($override_values);
+			$this->set_override_values($override_values);
 		}
 
-		if(PTS_IS_CLIENT && is_file($this->get_install_dir() . 'pts-install.xml'))
+		if($normal_init && PTS_IS_CLIENT && $this->identifier != null)
 		{
-			$this->test_installation = new pts_installed_test($this);
+			if(!isset(self::$test_installation_cache[$identifier]))
+			{
+				self::$test_installation_cache[$identifier] = new pts_installed_test($this);
+			}
+
+			$this->test_installation = &self::$test_installation_cache[$identifier];
 		}
 	}
+	public function validate()
+	{
+		$dom = new DOMDocument();
+		$dom->loadXML($this->get_xml());
+		return $dom->schemaValidate(pts_openbenchmarking::openbenchmarking_standards_path() . 'schemas/test-profile.xsd');
+	}
 	public static function is_test_profile($identifier)
 	{
-		$identifier = pts_openbenchmarking::evaluate_string_to_qualifier($identifier, true, 'test');
+		if(!is_file(PTS_TEST_PROFILE_PATH . $identifier . '/test-definition.xml'))
+		{
+			$identifier = pts_openbenchmarking::evaluate_string_to_qualifier($identifier, true, 'test');
+		}
 		return $identifier != false && is_file(PTS_TEST_PROFILE_PATH . $identifier . '/test-definition.xml') ? $identifier : false;
 	}
 	public function get_resource_dir()
 	{
 		return PTS_TEST_PROFILE_PATH . $this->identifier . '/';
 	}
-	public function get_override_values()
+	public function get_override_values($as_string = false)
 	{
-		return $this->xml_parser->getOverrideValues();
+		if($as_string)
+		{
+			$o = $this->overrides;
+			foreach($o as $x => &$y)
+			{
+				$y = $x . '=' . $y;
+			}
+
+			return implode(';', $o);
+
+		}
+		else
+		{
+			return $this->overrides;
+		}
 	}
 	public function set_override_values($override_values)
 	{
 		if(is_array($override_values))
 		{
-			$this->xml_parser->overrideXMLValues($override_values);
+			foreach($override_values as $xml_tag => $value)
+			{
+				$this->xs($xml_tag, $value);
+			}
 		}
 	}
 	public function get_download_size($include_extensions = true, $divider = 1048576)
 	{
 		$estimated_size = 0;
 
-		foreach(pts_test_install_request::read_download_object_list($this->identifier) as $download_object)
+		if(PTS_IS_CLIENT)
+		{
+			$downloads = pts_test_install_request::read_download_object_list($this->identifier);
+		}
+		else
+		{
+			$downloads = $this->get_downloads();
+		}
+
+		foreach($downloads as $download_object)
 		{
 			$estimated_size += $download_object->get_filesize();
 		}
@@ -111,7 +152,7 @@ class pts_test_profile extends pts_test_profile_parser
 			{
 				if(!in_array($extended_test, $extensions))
 				{
-					array_push($extensions, $extended_test);
+					$extensions[] = $extended_test;
 				}
 
 				$extended_test = new pts_test_profile_parser($extended_test);
@@ -122,33 +163,34 @@ class pts_test_profile extends pts_test_profile_parser
 
 		return $extensions;
 	}
-	public function get_dependency_names()
-	{
-		$dependency_names = array();
-		$exdep_generic_parser = new pts_exdep_generic_parser();
-
-		foreach($this->get_dependencies() as $dependency)
-		{
-			if($exdep_generic_parser->is_package($dependency))
-			{
-				$package_data = $exdep_generic_parser->get_package_data($dependency);
-				array_push($dependency_names, $package_data['title']);
-				break;
-			}
-		}
-
-		return $dependency_names;
-	}
 	public function get_times_to_run()
 	{
 		$times_to_run = parent::get_times_to_run();
 
+		if(!PTS_IS_CLIENT)
+		{
+			return $times_to_run;
+		}
+
+		if(($force_runs_multiple = pts_client::read_env('FORCE_TIMES_TO_RUN_MULTIPLE')) && is_numeric($force_runs_multiple) && $force_runs_multiple > 1 && $this->get_estimated_run_time() < (60 * 60 * 2))
+		{
+			$times_to_run *= $force_runs_multiple;
+		}
+
 		if(($force_runs = pts_client::read_env('FORCE_TIMES_TO_RUN')) && is_numeric($force_runs))
 		{
 			$times_to_run = $force_runs;
 		}
 
-		if(($force_runs = pts_client::read_env('FORCE_MIN_TIMES_TO_RUN')) && is_numeric($force_runs) && $force_runs > $times_to_run)
+		if(($force_min_cutoff = pts_client::read_env('FORCE_MIN_TIMES_TO_RUN_CUTOFF')) == false || ($this->get_estimated_run_time() > 0 && ($this->get_estimated_run_time() / 60) < $force_min_cutoff))
+		{
+			if(($force_runs = pts_client::read_env('FORCE_MIN_TIMES_TO_RUN')) && is_numeric($force_runs) && $force_runs > $times_to_run)
+			{
+				$times_to_run = $force_runs;
+			}
+		}
+
+		if(($force_runs = pts_client::read_env('FORCE_ABSOLUTE_MIN_TIMES_TO_RUN')) && is_numeric($force_runs) && $force_runs > $times_to_run)
 		{
 			$times_to_run = $force_runs;
 		}
@@ -165,6 +207,11 @@ class pts_test_profile extends pts_test_profile_parser
 	public function get_estimated_run_time()
 	{
 		// get estimated run-time (in seconds)
+		if($this->no_fallbacks_on_null)
+		{
+			return parent::get_estimated_run_time();
+		}
+
 		if($this->test_installation != false && is_numeric($this->test_installation->get_average_run_time()) && $this->test_installation->get_average_run_time() > 0)
 		{
 			$estimated_run_time = $this->test_installation->get_average_run_time();
@@ -183,6 +230,24 @@ class pts_test_profile extends pts_test_profile_parser
 
 		return $estimated_run_time;
 	}
+	public function get_estimated_install_time()
+	{
+		// get estimated install-time (in seconds)
+		$est_install_time = 0;
+		if($this->test_installation != false && is_numeric($this->test_installation->get_latest_install_time()) && $this->test_installation->get_latest_install_time() > 0)
+		{
+			$est_install_time = $this->test_installation->get_latest_install_time();
+		}
+
+		if($est_install_time == 0 && PTS_IS_CLIENT)
+		{
+			$identifier = explode('/', $this->get_identifier(false));
+			$repo_index = pts_openbenchmarking::read_repository_index($identifier[0]);
+			$est_install_time = isset($identifier[1]) && isset($repo_index['tests'][$identifier[1]]) && isset($repo_index['tests'][$identifier[1]]['average_install_time']) ? $repo_index['tests'][$identifier[1]]['average_install_time'] : 1;
+		}
+
+		return ceil($est_install_time);
+	}
 	public function is_supported($report_warnings = true)
 	{
 		$test_supported = true;
@@ -199,7 +264,7 @@ class pts_test_profile extends pts_test_profile_parser
 		}
 		else if($this->is_test_platform_supported() == false)
 		{
-			PTS_IS_CLIENT && $report_warnings && pts_client::$display->test_run_error($this->get_identifier() . ' is not supported by this operating system: ' . phodevi::operating_system());
+			PTS_IS_CLIENT && $report_warnings && pts_client::$display->test_run_error($this->get_identifier() . ' is not supported by this operating system: ' . phodevi::os_under_test());
 			$test_supported = false;
 		}
 		else if($this->is_core_version_supported() == false)
@@ -258,15 +323,8 @@ class pts_test_profile extends pts_test_profile_parser
 	public function is_test_architecture_supported()
 	{
 		// Check if the system's architecture is supported by a test
-		$supported = true;
 		$archs = $this->get_supported_architectures();
-
-		if(!empty($archs))
-		{
-			$supported = phodevi::cpu_arch_compatible($archs);
-		}
-
-		return $supported;
+		return !empty($archs) ? phodevi::cpu_arch_compatible($archs) : true;
 	}
 	public function is_core_version_supported()
 	{
@@ -283,7 +341,7 @@ class pts_test_profile extends pts_test_profile_parser
 
 		$platforms = $this->get_supported_platforms();
 
-		if(!empty($platforms) && !in_array(phodevi::operating_system(), $platforms))
+		if(!empty($platforms) && !in_array(phodevi::os_under_test(), $platforms))
 		{
 			if(phodevi::is_bsd() && in_array('Linux', $platforms) && (pts_client::executable_in_path('kldstat') && strpos(shell_exec('kldstat -n linux 2>&1'), 'linux.ko') != false))
 			{
@@ -304,16 +362,17 @@ class pts_test_profile extends pts_test_profile_parser
 
 		return $supported;
 	}
-	public static function generate_comparison_hash($test_identifier, $arguments, $attributes = null, $version = null)
+	public static function generate_comparison_hash($test_identifier, $arguments, $attributes = null, $version = null, $result_scale = null, $raw_output = true)
 	{
 		$hash_table = array(
 		$test_identifier,
 		trim($arguments),
 		trim($attributes),
-		$version
+		trim($version),
+		trim($result_scale)
 		);
 
-		return sha1(implode(',', $hash_table), true);
+		return sha1(implode(',', $hash_table), $raw_output);
 	}
 	public function get_test_executable_dir()
 	{
@@ -321,7 +380,7 @@ class pts_test_profile extends pts_test_profile_parser
 		$test_dir = $this->get_install_dir();
 		$execute_binary = $this->get_test_executable();
 
-		if(is_executable($test_dir . $execute_binary) || (phodevi::is_windows() && is_file($test_dir . $execute_binary)))
+		if(is_file($test_dir . $execute_binary)) // previously was: (is_executable($test_dir . $execute_binary) || (phodevi::is_windows() && is_file($test_dir . $execute_binary))
 		{
 			$to_execute = $test_dir;
 		}
@@ -332,9 +391,19 @@ class pts_test_profile extends pts_test_profile_parser
 	{
 		return is_file($this->get_install_dir() . 'pts-install.xml');
 	}
+	public function get_test_executable()
+	{
+		$exe = parent::get_test_executable();
+		if(!is_file($this->get_install_dir() . $exe) && phodevi::is_windows() && is_file($this->get_install_dir() . $exe . '.bat'))
+		{
+			$exe .= '.bat';
+		}
+
+		return $exe;
+	}
 	public function get_install_dir()
 	{
-		return pts_client::test_install_root_path() . $this->identifier . '/';
+		return pts_client::test_install_root_path() . $this->identifier . DIRECTORY_SEPARATOR;
 	}
 	public function get_installer_checksum()
 	{
@@ -343,7 +412,7 @@ class pts_test_profile extends pts_test_profile_parser
 	public function get_file_installer()
 	{
 		$test_resources_location = $this->get_resource_dir();
-		$os_postfix = '_' . strtolower(phodevi::operating_system());
+		$os_postfix = '_' . strtolower(phodevi::os_under_test());
 
 		if(is_file($test_resources_location . 'install' . $os_postfix . '.sh'))
 		{
@@ -366,7 +435,20 @@ class pts_test_profile extends pts_test_profile_parser
 	}
 	public function get_file_parser_spec()
 	{
-		return is_file($this->get_resource_dir() . 'results-definition.xml') ? $this->get_resource_dir() . 'results-definition.xml' : false;
+		$spec = is_file($this->get_resource_dir() . 'results-definition.xml') ? $this->get_resource_dir() . 'results-definition.xml' : false;
+
+		if(!$spec)
+		{
+			$extends = $this->get_test_extension();
+
+			if(!empty($extends))
+			{
+				$test_profile = new pts_test_profile($extends);
+				$spec = $test_profile->get_file_parser_spec();
+			}
+		}
+
+		return $spec;
 	}
 	public function extended_test_profiles()
 	{
@@ -376,7 +458,7 @@ class pts_test_profile extends pts_test_profile_parser
 		foreach(array_unique(array_reverse($this->get_test_extensions_recursive())) as $extended_test)
 		{
 			$test_profile = new pts_test_profile($extended_test);
-			array_push($test_profiles, $test_profile);
+			$test_profiles[] = $test_profile;
 		}
 
 		return $test_profiles;
@@ -384,21 +466,114 @@ class pts_test_profile extends pts_test_profile_parser
 	public function needs_updated_install()
 	{
 		// Checks if test needs updating
-		// || $this->test_installation->get_installed_system_identifier() != phodevi::system_id_string()
-		return $this->test_installation == false || $this->get_test_profile_version() != $this->test_installation->get_installed_version() || $this->get_installer_checksum() != $this->test_installation->get_installed_checksum() || (pts_c::$test_flags & pts_c::force_install);
+		return $this->test_installation == false || $this->get_test_profile_version() != $this->test_installation->get_installed_version() || $this->get_installer_checksum() != $this->test_installation->get_installed_checksum() || $this->test_installation->get_system_hash() != phodevi::system_id_string();
 	}
 	public function to_json()
 	{
-		$file = $this->xml_parser->getFileLocation();
-
-		if(is_file($file))
+		$file = $this->get_xml();
+		$file = str_replace(array("\n", "\r", "\t"), '', $file);
+		$file = trim(str_replace('"', "'", $file));
+		$simple_xml = simplexml_load_string($file);
+		return json_encode($simple_xml);
+	}
+	public function get_downloads()
+	{
+		$download_xml_file = $this->get_file_download_spec();
+		$downloads = array();
+		if($download_xml_file != null)
 		{
-			$file = file_get_contents($file);
-			$file = str_replace(array("\n", "\r", "\t"), '', $file);
-			$file = trim(str_replace('"', "'", $file));
-			$simple_xml = simplexml_load_string($file);
-			return json_encode($simple_xml);
+			$xml = simplexml_load_file($download_xml_file, 'SimpleXMLElement', LIBXML_COMPACT | LIBXML_PARSEHUGE);
+
+			if($xml->Downloads && $xml->Downloads->Package)
+			{
+				foreach($xml->Downloads->Package as $pkg)
+				{
+					$pkg_url = isset($pkg->URL) ? $pkg->URL->__toString() : null;
+					$pkg_md5 = isset($pkg->MD5) ? $pkg->MD5->__toString() : null;
+					$pkg_sha256 = isset($pkg->SHA256) ? $pkg->SHA256->__toString() : null;
+					$pkg_filename = isset($pkg->FileName) ? $pkg->FileName->__toString() : null;
+					$pkg_filesize = isset($pkg->FileSize) ? $pkg->FileSize->__toString() : null;
+					$pkg_architecture = isset($pkg->ArchitectureSpecific) ? $pkg->ArchitectureSpecific->__toString() : null;
+					$pkg_platforms = isset($pkg->PlatformSpecific) ? $pkg->PlatformSpecific->__toString() : null;
+					$is_optional = isset($pkg->Optional) ? $pkg->Optional->__toString() : null;
+					$downloads[] = new pts_test_file_download($pkg_url, $pkg_filename, $pkg_filesize, $pkg_md5, $pkg_sha256, $pkg_platforms, $pkg_architecture, $is_optional);
+				}
+			}
 		}
+		return $downloads;
+	}
+	public function get_results_definition($limit = null)
+	{
+		$results_definition_file = $this->get_file_parser_spec();
+		$results_definition = new pts_test_profile_results_definition();
+		if($results_definition_file != null)
+		{
+			$xml = simplexml_load_file($results_definition_file, 'SimpleXMLElement', LIBXML_COMPACT | LIBXML_PARSEHUGE);
+
+			if($xml->SystemMonitor && ($limit == null || $limit == 'SystemMonitor'))
+			{
+				foreach($xml->SystemMonitor as $i)
+				{
+					$s = isset($i->Sensor) ? $i->Sensor->__toString() : null;
+					$p = isset($i->PollingFrequency) ? $i->PollingFrequency->__toString() : null;
+					$r = isset($i->Report) ? $i->Report->__toString() : null;
+					$results_definition->add_system_monitor_definition($s, $p, $r);
+				}
+			}
+			if($xml->ExtraData && ($limit == null || $limit == 'ExtraData'))
+			{
+				foreach($xml->ExtraData as $i)
+				{
+					$results_definition->add_extra_data_definition((isset($i->Identifier) ? $i->Identifier->__toString() : null));
+				}
+			}
+			if($xml->ImageParser && ($limit == null || $limit == 'ImageParser'))
+			{
+				foreach($xml->ImageParser as $i)
+				{
+					$s = isset($i->SourceImage) ? $i->SourceImage->__toString() : null;
+					$m = isset($i->MatchToTestArguments) ? $i->MatchToTestArguments->__toString() : null;
+					$x = isset($i->ImageX) ? $i->ImageX->__toString() : null;
+					$y = isset($i->ImageY) ? $i->ImageY->__toString() : null;
+					$w = isset($i->ImageWidth) ? $i->ImageWidth->__toString() : null;
+					$h = isset($i->ImageHeight) ? $i->ImageHeight->__toString() : null;
+					$results_definition->add_image_parser_definition($s, $m, $x, $y, $w, $h);
+				}
+			}
+			if($xml->ResultsParser && ($limit == null || $limit == 'ResultsParser'))
+			{
+				foreach($xml->ResultsParser as $i)
+				{
+					$ot = isset($i->OutputTemplate) ? $i->OutputTemplate->__toString() : null;
+					$mtta = isset($i->MatchToTestArguments) ? $i->MatchToTestArguments->__toString() : null;
+					$rk = isset($i->ResultKey) ? $i->ResultKey->__toString() : null;
+					$lh = isset($i->LineHint) ? $i->LineHint->__toString() : null;
+					$lbh = isset($i->LineBeforeHint) ? $i->LineBeforeHint->__toString() : null;
+					$lah = isset($i->LineAfterHint) ? $i->LineAfterHint->__toString() : null;
+					$rbs = isset($i->ResultBeforeString) ? $i->ResultBeforeString->__toString() : null;
+					$ras = isset($i->ResultAfterString) ? $i->ResultAfterString->__toString() : null;
+					$sfr = isset($i->StripFromResult) ? $i->StripFromResult->__toString() : null;
+					$srp = isset($i->StripResultPostfix) ? $i->StripResultPostfix->__toString() : null;
+					$mm = isset($i->MultiMatch) ? $i->MultiMatch->__toString() : null;
+					$drb = isset($i->DivideResultBy) ? $i->DivideResultBy->__toString() : null;
+					$drd = isset($i->DivideResultDivisor) ? $i->DivideResultDivisor->__toString() : null;
+					$mrb = isset($i->MultiplyResultBy) ? $i->MultiplyResultBy->__toString() : null;
+					$rs = isset($i->ResultScale) ? $i->ResultScale->__toString() : null;
+					$rpro = isset($i->ResultProportion) ? $i->ResultProportion->__toString() : null;
+					$rpre = isset($i->ResultPrecision) ? $i->ResultPrecision->__toString() : null;
+					$ad = isset($i->ArgumentsDescription) ? $i->ArgumentsDescription->__toString() : null;
+					$atad = isset($i->AppendToArgumentsDescription) ? $i->AppendToArgumentsDescription->__toString() : null;
+					$ff = isset($i->FileFormat) ? $i->FileFormat->__toString() : null;
+					$tcts = isset($i->TurnCharsToSpace) ? $i->TurnCharsToSpace->__toString() : null;
+					$dob = isset($i->DeleteOutputBefore) ? $i->DeleteOutputBefore->__toString() : null;
+					$doa = isset($i->DeleteOutputAfter) ? $i->DeleteOutputAfter->__toString() : null;
+					$df = isset($i->DisplayFormat) ? $i->DisplayFormat->__toString() : null;
+					$results_definition->add_result_parser_definition($ot, $mtta, $rk, $lh, $lbh, $lah, $rbs, $ras, $sfr, $srp, $mm, $drb, $mrb, $rs, $rpro, $rpre, $ad, $atad, $ff, $tcts, $dob, $doa, $df, $drd);
+				}
+			}
+		}
+
+		return $results_definition;
 	}
 }
 
