@@ -3,8 +3,8 @@
 /*
 	Phoronix Test Suite
 	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
-	Copyright (C) 2008 - 2014, Phoronix Media
-	Copyright (C) 2008 - 2014, Michael Larabel
+	Copyright (C) 2008 - 2021, Phoronix Media
+	Copyright (C) 2008 - 2021, Michael Larabel
 	phoronix-test-suite.php: The main code for initalizing the Phoronix Test Suite
 
 	This program is free software; you can redistribute it and/or modify
@@ -34,7 +34,7 @@ if(!defined('PTS_MODE'))
 }
 
 // Any PHP default memory limit should be fine for PTS, until you run image quality comparison tests that begins to consume memory
-ini_set('memory_limit', '256M');
+ini_set('memory_limit', '1024M');
 
 if(getenv('PTS_MODE') == 'CLIENT' && ini_get('open_basedir') != false)
 {
@@ -68,6 +68,7 @@ if(getenv('PTS_MODE') == 'CLIENT' && ini_get('open_basedir') != false)
 	else
 	{
 		echo PHP_EOL . 'NOTICE: The php.ini configuration is using the "open_basedir" directive, which may prevent some parts of the Phoronix Test Suite from working. See the Phoronix Test Suite documentation for more details and to disable this setting.' . PHP_EOL;
+		sleep(1);
 	}
 }
 
@@ -93,51 +94,25 @@ if(QUICK_START == false)
 {
 	pts_client::program_requirement_checks(true);
 }
+
 pts_client::init(); // Initalize the Phoronix Test Suite (pts-core) client
 $pass_args = array();
 
-if(is_file(PTS_PATH . 'pts-core/commands/' . $sent_command . '.php') == false)
-{
-	$replaced = false;
-
-	if(pts_module::valid_run_command($sent_command))
-	{
-		$replaced = true;
-	}
-	else if(isset($argv[1]) && strpos($argv[1], '.openbenchmarking') !== false && is_readable($argv[1]))
-	{
-		$sent_command = 'openbenchmarking_launcher';
-		$argv[2] = $argv[1];
-		$argc = 3;
-		$replaced = true;
-	}
-	else
-	{
-		$aliases = pts_storage_object::read_from_file(PTS_TEMP_STORAGE, 'command_alias_list');
-		if($aliases == null)
-		{
-			$aliases = pts_documentation::client_commands_aliases();
-		}
-
-		if(isset($aliases[$sent_command]))
-		{
-			$sent_command = $aliases[$sent_command];
-			$replaced = true;
-		}
-	}
-
-	if($replaced == false)
-	{
-		// Show help command, since there are no valid commands
-		$sent_command = 'help';
-	}
-}
+pts_client::handle_sent_command($sent_command, $argv, $argc);
 
 
 pts_define('PTS_USER_LOCK', function_exists('posix_getpid') ? PTS_USER_PATH . 'run-lock-' . posix_getpid() : tempnam(PTS_USER_PATH, 'run-lock-'));
 
 if(QUICK_START == false)
 {
+	// Cleanup old / expired runlocks XXX expire if this can eventually be removed
+	foreach(pts_file_io::glob(PTS_USER_PATH . 'run-lock-*') as $possible_run_lock)
+	{
+		if(!pts_client::is_locked($possible_run_lock))
+		{
+			pts_file_io::unlink($possible_run_lock);
+		}
+	}
 	if(pts_client::create_lock(PTS_USER_LOCK) == false)
 	{
 		//trigger_error('It appears that the Phoronix Test Suite is already running.' . PHP_EOL . 'For proper results, only run one instance at a time.', E_USER_WARNING);
@@ -155,13 +130,20 @@ if(QUICK_START == false)
 // Read passed arguments
 for($i = 2; $i < $argc && isset($argv[$i]); $i++)
 {
-	array_push($pass_args, $argv[$i]);
+	if(($x = strpos($argv[$i], '=')) !== false && substr($argv[$i], 0, $x) == strtoupper(substr($argv[$i], 0, $x)))
+	{
+		// Likely trying to pass an environment variable, so just go ahead and set it...
+		putenv($argv[$i]);
+	}
+	else
+	{
+		$pass_args[] = $argv[$i];
+	}
 }
 
 if(QUICK_START == false)
 {
 	pts_client::user_agreement_check($sent_command);
-	pts_client::user_hardware_software_reporting();
 
 	// OpenBenchmarking.org
 	pts_openbenchmarking::refresh_repository_lists();
